<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * DataGrid Komponente (Datentabelle).
 *
 * Diese Komponente stellt tabellarische Daten dar.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiDataGrid extends GhGuiFormComponent
{
  /**
   * Keine Auswahl (es können keine Zeilen ausgewählt werden).
   */
  const SELECTION_NONE = 'none';

  /**
   * Einfachauswahl (es kann immer nur eine einzelne Zeile ausgewählt werden).
   */
  const SELECTION_SINGLE = 'single';

  /**
   * Mehrfachauswahl (mehrere Zeilen können durch Anklicken ausgewählt und abgewählt werden).
   */
  const SELECTION_MULTIPLE = 'multiple';

  /**
   * Mehrfachauswahl (mehrere Zeilen können mit der Umschalt- und der Strg-Taste ausgewählt werden).
   */
  const SELECTION_EXTENDED = 'extended';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.DataGrid';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dojo.data.ObjectStore', 'dojo.store.Memory');

  /**
   * Eigenschaften für das aus dieser Komponente erzeugte Widget Objekt.
   * @var array
   */
  protected $_widgetProperties = array('query' => '{_id:"*"}', 'autoWidth' => false, 'columnReordering' => true);

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onSelectionChanged');

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiDataGrid');

  /**
   * CSS Angaben für den Haupt-DOM-Knoten dieser Komponente.
   * Diese Angaben werden als 'style' Attribut bei der Ausgabe des HTML-Codes
   * den Attributen der Komponente hinzugefügt.
   *
   * Das DataGrid nutzt standardmäßig den gesamten zur Verfügung stehenden
   * Platz.
   *
   * @var array
   */
  protected $_styles = array('width' => '100%', 'height' => '100%');

  /**
   * Spaltendefinitionen des DataGrid (Spalten).
   * Dies sind Instanzen der Klasse GhGuiDataGridColumn.
   * @var array
   */
  protected $_columns = array();

  /**
   * Daten des DataGrid (Zeilen).
   * @var array
   */
  protected $_rows = array();

  /**
   * Auswahlmodus (GhGuiDataGrid::NONE, GhGuiDataGrid::SINGLE, GhGuiDataGrid::MULTIPLE oder GhGuiDataGrid::EXTENDED).
   * @var string
   */
  protected $_selectionMode = self::SELECTION_EXTENDED;

  /**
   * Eigenschaft der Datenobjekte für den Wert des Formularfeldes.
   * @var string
   */
  protected $_valueProperty = 'ID';

  /**
   * Werte der vorausgewählten DataGrid Zeilen.
   * @var string
   */
  protected $_values;

  /**
   * Legt fest, ob die Attribute 'name' und 'value' für den HTML-Knoten erzeugt
   * werden.
   * @var boolean
   */
  protected $_createNameAndValueAttributes = false;

  /**
   * Legt fest, ob die Eigenschaften 'name' und 'value' für das Javascript
   * Widget erzeugt werden.
   * @var boolean
   */
  protected $_createNameAndValueWidgetProperties = false;

  /**
   * Konstruktor.
   *
   * Optional kann der Name des Eingabefeldes festgelegt werden (siehe
   * setName()), sowie der Auswahlmodus (siehe setSelectionMode()), die
   * auto-submit Funktion (siehe setAutoSubmit()) und die Objekteigenschaft,
   * die für den Wert des versteckten Formularfelds verwendet wird (siehe
   * setValueProperty()).
   *
   * @param string  $name          Name des Formularfeldes
   * @param string  $selectionMode Auswahlmodus
   * @param boolean $autoSubmit    true: auto-submit, false: kein auto-submit
   * @param array   $valueProperty Objekteigenschaft
   */
  public function __construct($name = null, $selectionMode = null, $autoSubmit = null, $valueProperty = null)
  {
    parent::__construct($name, null, null, $autoSubmit);

    if($selectionMode !== null)
      $this->setSelectionMode($selectionMode);

    if($valueProperty !== null)
      $this->setValueProperty($valueProperty);
  }

  /**
   * Fügt eine neue Spalte zur Tabelle hinzu.
   *
   * Spalten ohne Beschriftung (Spaltentitel) werden nicht im Data-Grid
   * angezeigt. Solche Spalten hinzuzufügen macht Sinn, wenn man den Wert der
   * Spalte als Wert für das Formularfeld nutzen möchte (siehe
   * setValueProperty()).
   *
   * Zusätzlich zu der Objekteigenschaft für diese Spalte, der Beschriftung
   * und der Spaltenbreite kann ein Formatierer für die Zellen dieser Spalte
   * übergeben werden.
   * Der Formatierer ist eine Javascript Funktion, die als Parameter die
   * Daten der Tabellenzelle erhält und einen String mit dem darzustellenden
   * Inhalt zurückliefert.
   *
   * Als vordefinierte Formatierer können die Konstanten der Klasse
   * [classes]GhGuiFormatter[/classes] verwendet werden.
   *
   * @param string         $property      Objekteigenschaft für die Werte dieser Spalte
   * @param string         $caption       Beschriftung (Spaltentitel)
   * @param string         $width         Spaltenbreite (Pixel oder Prozentangabe)
   * @param GhGuiFormatter $cellFormatter Zellenformatierer (Javascript-Funktion)
   * @return GhGuiDataGridColumn die neu hinzugefügte Spalte
   */
  public function addColumn($property, $caption = null, $width = null, $cellFormatter = null)
  {
    if($property instanceof GhGuiDataGridColumn)
      $column = $property;
    else
      $column = new GhGuiDataGridColumn($property, $caption, $width, $cellFormatter);
    $this->_columns[] = $column;

    return $column;
  }

  /**
   * Liefert die Spalten (inkl. der versteckten Spalten) der Tabelle.
   *
   * @return array
   */
  public function getColumns()
  {
    return $this->_columns;
  }

  /**
   * Fügt eine Zeile mit Daten zur Tabelle hinzu.
   * Die Daten müssen entweder als Objekt mit entsprechenden Eigenschaften
   * oder als Array mit entsprechenden Schlüsseln vorliegen.
   *
   * @throws GhGuiException falls $data weder ein Objekt noch ein Array ist
   *
   * @param object|array $data Datenobjekt oder -array
   * @return GhGuiDataGrid liefert das DataGrid zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addRow($data)
  {
    if(is_object($data))
      $data = clone $data;
    elseif(is_array($data))
      $data = (object)$data;
    else
      throw new GhGuiException('Only objects and arrays may be added as data to a GhGuiDataGrid');

    $data->_id = GhGui::createId();
    $this->_rows[] = $data;

    return $this;
  }

  /**
   * Fügt mehrere Zeilen mit Daten zur Tabelle hinzu.
   *
   * @throws GhGuiException falls $rows kein Array mit Objekten oder Arrays ist
   *
   * @param array $rows ein Array von Datenobjekten oder -arrays
   * @return GhGuiDataGrid liefert das DataGrid zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addRows($rows)
  {
    foreach($rows as $row)
      $this->addRow($row);

    return $this;
  }

  /**
   * Liefert die Zeilen (Daten) der Tabelle.
   *
   * @return array
   */
  public function getRows()
  {
    return $this->_rows;
  }

  /**
   * Setzt die Datenobjekteigenschaft, die den Wert für das Formularfeld
   * enthält.
   *
   * @param string $property Eigenschaft der Datenobjekte
   * @return GhGuiDataGrid liefert das DataGrid zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValueProperty($property)
  {
    $this->_valueProperty = $property;

    return $this;
  }

  /**
   * Liefert die Datenobjekteigenschaft, die den Wert für das Formularfeld
   * enthält.
   *
   * @return string Datenobjekteigenschaft
   */
  public function getValueProperty()
  {
    return $this->_valueProperty;
  }

  /**
   * Serzt den Wert dieses Formularfeldes.
   *
   * @param string $value Wert für das Formularfeld
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValue($value)
  {
    // Keine Widget Eigenschaft verwenden:
    $this->_values = $value;

    return $this;
  }

  /**
   * Liefert den Wert des Formularfeldes.
   *
   * @return string
   */
  public function getValue()
  {
    return $this->_values;
  }

  /**
   * Setzt den Text, der statt der Tabelle angezeigt wird, wenn keine Daten
   * vorliegen (die Tabelle also leer ist).
   *
   * @param string $message Text für leere Tabelle
   * @return GhGuiDataGrid liefert das DataGrid zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setNoDataMessage($message)
  {
    $this->setWidgetProperty('noDataMessage', $message);

    return $this;
  }

  /**
   * Liefert den Text, der statt der Tabelle angezeigt wird, wenn keine Daten
   * vorliegen (die Tabelle also leer ist).
   *
   * @return string Text für leere Tabelle
   */
  public function getNoDataMessage()
  {
    return $this->getWidgetProperty('noDataMessage');
  }

  /**
   * Legt den Auswahlmodus für Tabellenzeilen fest.
   *
   * <dl>
   * <dt><tt>GhGuiDataGrid::NONE</tt></dt>
   * <dd>Keine Auswahl (es können keine Zeilen ausgewählt werden).</dd>
   * <dt><tt>GhGuiDataGrid::SINGLE</tt></dt>
   * <dd>Einfachauswahl (es kann immer nur eine einzelne Zeile ausgewählt werden).</dd>
   * <dt><tt>GhGuiDataGrid::MULTIPLE</tt></dt>
   * <dd>Mehrfachauswahl (mehrere Zeilen können durch Anklicken ausgewählt und abgewählt werden).</dd>
   * <dt><tt>GhGuiDataGrid::EXTENDED</tt></dt>
   * <dd>Mehrfachauswahl (mehrere Zeilen können mit der Umschalt- und der Strg-Taste ausgewählt werden).</dd>
   * </dl>
   *
   * @param string $mode Auswahlmodus (eine der Konstanten GhGuiDataGrid::NONE,
   *   GhGuiDataGrid::SINGLE, GhGuiDataGrid::MULTIPLE oder
   *   GhGuiDataGrid::EXTENDED)
   * @return GhGuiDataGrid liefert das DataGrid zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setSelectionMode($mode)
  {
    $this->setWidgetProperty('selectionMode', $mode);

    return $this;
  }

  /**
   * Liefert den Auswahlmodus für Tabellenzeilen.
   *
   * @return string
   */
  public function getSelectionMode()
  {
    return $this->getWidgetProperty('selectionMode');
  }

  /**
   * Legt fest, ob das DataGrid das Kontextmenu des Browsers angzeigt.
   *
   * In der Voreinstellung verwendet das DataGrid das Kontextmenu des Browsers.
   * Über diese Methode kann dieses Verhalten abgeschaltet werden, so dass sich
   * das Kontextmenu des Datagrid überschreiben lässt. Sofern nicht über
   * Javascript ein eigenes Kontextmenu für das DataGrid erzeugt wird, wird
   * dann gar kein Kontextmenu angezeigt.
   *
   * @param boolean $useBrowserContextMenu true: Browser-Kontextmenu verwenden, false: Kontextmenu des DataGrid verwenden
   */
  public function setUseBrowserContextMenu($useBrowserContextMenu)
  {
    $this->setWidgetProperty('useBrowserContextMenu', $useBrowserContextMenu ? true : false);
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   *
   * Das DataGrid registriert alle benötigten versteckten Formularfelder
   * (hidden fields) bei seinem Formular-Container.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $name = $this->getName();

    if(!$name)
      $name = $this->getId();

    $form = $this->getForm();

    if(!$form)
      throw new GhGuiException('The GhGuiDataGrid has not been added to a GhGuiForm');

    $html = '';

    $valueProperty = $this->getValueProperty();
    $currentSelection = $this->getCurrentValue();

    if(!is_array($currentSelection))
    {
      if($currentSelection)
        $currentSelection = array($currentSelection);
      else
        $currentSelection = array();
    }

    $html .= GhGui::createHtml('option', array('value' => ''));

    foreach($this->getRows() as $data)
    {
      if(isset($data->$valueProperty))
      {
        $attributes = array('id' => 'option_' . $data->_id, 'value' => $data->$valueProperty);
        if(in_array($data->$valueProperty, $currentSelection))
          $attributes['selected'] = 'selected';
        $html .= GhGui::createHtml('option', $attributes, $data->_id);
      }
    }

    $attributes = array('id' => 'selection_' . $this->getId(), 'style' => 'display:none;');
    $selectionMode = $this->getSelectionMode();
    if($selectionMode == self::SELECTION_SINGLE || $selectionMode == self::SELECTION_NONE)
      $attributes['name'] = $name;
    else
    {
      $name .= '[]';
      $attributes['name'] = $name;
      $attributes['multiple'] = 'multiple';
    }
    $html = GhGui::createHtml('select', $attributes, $html);

    $form->addHiddenField($name, null, 'selection_' . $this->getId(), $html);

    return true;
  }

  /**
   * Liefert den Javascript Code für diese Komponente.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    $javascript = 'var store_' . $this->getId() . '=new dojo.data.ObjectStore({objectStore:new dojo.store.Memory({idProperty:"_id",data:[';

    $first = true;

    foreach($this->_rows as $data)
    {
      // JSON Code für die Zeile hinzufügen:
      if(!$first)
        $javascript .= ',';

      $javascript .= GhGui::$eol . $this->_getRowJson($data);
      $first = false;
    }

    if(!empty($this->_rows))
      $javascript .= GhGui::$eol;

    $javascript .= ']})});' . GhGui::$eol;

    // Wenn keine Breite für mind. eine Spalte angegeben ist werden die Spalten gleichmäßig verteilt:
    $fixedWidth = false;

    if(!empty($this->_columns))
    {
      $fixedWidth = 100.0 / count($this->_columns);

      foreach($this->_columns as $column)/* @var GhGuiDataGridColumn $column */
      {
        if($column->getWidth())
        {
          $fixedWidth = false;
          break;
        }
      }
    }

    // Spalten:
    $structure = '[';
    $lastColumn = count($this->_columns) - 1;
    $first = true;

    for($i=0; $i<=$lastColumn; $i++)
    {
      $column = $this->_columns[$i];/* @var GhGuiDataGridColumn $column */

      if(!$column->getVisible())
        continue;

      if(!$first)
        $structure .= ',';

      $first = false;

      $caption = $column->getCaption();

      if(!$caption)
        $caption = $column->getProperty();

      $structure .= GhGui::$eol . '{field:"' . $column->getProperty() . '",name:"' . $caption . '"';

      $width = $column->getWidth();

      // Gleichmäßige Breite?
      if($fixedWidth)
        $width = $fixedWidth . '%';
      // Letzte Spalte ohne Breite? Dann Rest-Breite nutzen (Tabelle ausdehnen):
      elseif(!$width && $i == $lastColumn)
        $width = '100%';

      // Breite setzen:
      if($width)
        $structure .= ',width:"' . $width . '"';

      if($column->getCellStyle())
        $structure .= ',cellStyles:"' . $column->getCellStyle() . '"';

      if($column->getStyle())
        $structure .= ',headerStyles:"' . $column->getStyle() . '"';

      $structure .= ',formatter:function(data){var ret=';
      $formatter = $column->getCellFormatter();

      if($formatter)
        $structure .= $formatter->getJavascript($this) . '(data)';
      else
        $structure .= 'data';

      $structure .= ';if(ret!=undefined)return "<div class=\"GhGuiDataGridCellContent\">"+ret+"</div>";else return "<div class=\"GhGuiDataGridCellContent\"></div>";}';
      $structure .= '}';
    }

    if(!empty($this->_columns))
      $structure .= GhGui::$eol;

    $structure .= ']' . GhGui::$eol;

    // Widget erzeugen:
    $properties = $this->_getWidgetCreationProperties();
    $properties['store'] = 'store_' . $this->getId();
    $properties['structure'] = $structure;

    $javascript .= $this->_getWidgetCreationJavascript(null, $properties);

    return $javascript;
  }

  /**
   * Liefert den Javascript Code für die Event-Handler dieser Komponente.
   *
   * @return string
   */
  protected function _getEventHandlersJavascript()
  {
    $javascript = '';

    // Bei Focus einer Zelle die CSS Klasse 'ghGuiDataGridRowFocused' zur ganzen Zeile hinzufügen:
    $javascript .= 'dojo.connect(' . $this->getJavascriptWidgetVariable() . ',"onFocus",function(){dojo.addClass(' . $this->getJavascriptWidgetVariable() . '.domNode,"ghGuiDataGridFocused");});' . GhGui::$eol;
    $javascript .= 'dojo.connect(' . $this->getJavascriptWidgetVariable() . ',"onBlur",function(){dojo.removeClass(' . $this->getJavascriptWidgetVariable() . '.domNode,"ghGuiDataGridFocused");});' . GhGui::$eol;

    // Zeilen anhand des versteckten Formularfelds auswählen:
    $javascript .= 'greyhound.dojo.setDataGridSelection(' . $this->getJavascriptWidgetVariable() . ',dojo.byId("selection_' . $this->getId() . '"));' . GhGui::$eol;

    // Event-Handler für geänderte Auswahl:
    $javascript .= 'dojo.connect(' . $this->getJavascriptWidgetVariable() . ',"onSelectionChanged",function(){greyhound.dojo.updateDataGridSelectionHiddenField(' . $this->getJavascriptWidgetVariable() . ',dojo.byId("selection_' . $this->getId() . '"));});' . GhGui::$eol;

    // Event-Handler für Änderung der Sortierung (um die Auswahl mit der neuen Sortierung zu aktualisieren):
    // Idee: onChanged-Methode des DataGrid.selection Objekts leeren und nach der aktualisierung der Auswahl wieder setzen, um zu vermeiden,
    //       dass Selection-Events entstehen obwohl sich die Auswahl faktisch nicht ändert.
    $javascript .= 'dojo.connect(' . $this->getJavascriptWidgetVariable() . ',"onHeaderCellClick",function(){var onChanged=' . $this->getJavascriptWidgetVariable() . '.selection.onChanged;' . $this->getJavascriptWidgetVariable() . '.selection.onChanged = function(){};greyhound.dojo.setDataGridSelection(widget_' . $this->getId() . ',dojo.byId("selection_' . $this->getId() . '"));widget_' . $this->getId() . '.selection.onChanged=onChanged;onChanged=null;});' . GhGui::$eol;

    // Weitere Event-Handler:
    $javascript .= parent::_getEventHandlersJavascript();

    return $javascript;
  }

  /**
   * Liefert die Daten einer Zeile im JSON Format (für den Store des DataGrid).
   *
   * @param object $data Daten
   * @return string
   */
  protected function _getRowJson($data)
  {
    $json = '{_id:"' . $data->_id . '"';

    foreach($this->getColumns() as $column)/* @var GhGuiDataGridColumn $column */
    {
      $property = $column->getProperty();
      $json .= ',' . $property . ':';
      $quote = true;

      if(isset($data->$property))
        $value = $data->$property;
      else
        $value = null;
      /* @var mixed $value */

      switch($column->getType())
      {
        case GhGuiDataGridColumn::TYPE_DATE:
          if(!($value instanceof GhRpcDateTime))
            $value = new GhRpcDateTime($value);
          break;

        case GhGuiDataGridColumn::TYPE_FLOAT:
          if(is_numeric($value))
            $value = floatval($value);
          else
            $value = 0.0;

          $quote = false;
          break;

        case GhGuiDataGridColumn::TYPE_INT:
          if(is_numeric($value))
            $value = intval($value);
          else
            $value = 0;

          $quote = false;
          break;

        case GhGuiDataGridColumn::TYPE_STRING:
          $value = (string)$value;
          break;
      }

      if($value === null)
        $json .= 'null';
      else
      {
        if($value instanceof GhRpcDateTime)
          $json .= 'new Date(' . ($value->getTimeStamp() * 1000) . ')'; // javascript expects milliseconds
        elseif($quote)
          $json .= '"' . str_replace('"', '\\"', $value) . '"';
        else
          $json .= $value;
      }
    }

    $json .= '}';

    return $json;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'autoWidth':
      case 'columnReordering':
      case 'useBrowserContextMenu':
        return 'boolean';

      case 'noDataMessage':
      case 'selectionMode':
        return 'string';

      case 'query':
      case 'store':
        return 'object';

      case 'structure':
        return 'array';
    }

    return parent::__getWidgetPropertyType($property);
  }
}