<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2013-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * DropDown-Button Komponente (Schaltfläche).
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2013-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiDropDownButton extends GhGuiComboButton
{
  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.form.DropDownButton';

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array();

  /**
   * Auto-Submit.
   * @var boolean
   */
  protected $_autoSubmit = null;


  /**
   * Konstruktor.
   *
   * Optional kann die Beschriftung des Buttons angegeben werden, sowie die
   * auto-submit Funktion deaktiviert werden (siehe setAutoSubmit()). Bei
   * Buttons ist die auto-submit Funktion als Vorgabe eingeschaltet, so dass
   * ein Klick auf ein Button zum Absenden des Formulars führt.
   *
   * @param string  $caption    Beschriftung des Buttons
   * @param string  $name       Formularfeld-Name des Buttons
   * @param boolean $autoSubmit true: auto-submit, false: kein auto-submit
   */
  public function __construct($caption = null, $name = null, $autoSubmit = true)
  {
    parent::__construct($caption, $name, $autoSubmit);
  }

  /**
   * Legt fest, ob das Formular beim Ändern des Wertes automatisch abgeschickt wird.
   *
   * Falls dieser Wert auf true eingestellt wird, dann wird das Formular, in
   * dem sich diese Komponente befindet, beim Ändern des Wertes bzw. beim
   * Auswählen eines Wertes automatisch abgeschickt.
   *
   * @param boolean $autoSubmit true: Formular automatisch abschicken,
   *   false: Formular nicht automatisch abschicken
   * @return GhGuiDropDownButton liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setAutoSubmit($autoSubmit)
  {
    $this->_autoSubmit = $autoSubmit ? true : false;

    return $this;
  }

  /**
   * Liefert die auto-submit Einstellung.
   *
   * @return boolean
   */
  public function getAutoSubmit()
  {
    return $this->_autoSubmit;
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   *
   * Falls das Button einen Feldnamen erhalten hat registriert es ein
   * verstecktes Formularfeld mit diesem Namen bei seinem Formular-Container,
   * damit beim Absenden des Formulars der Name und Wert des Buttons mit
   * gesendet werden.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $name = $this->getName();

    if($name !== null && $this->getAutoSubmit())
    {
      if(!$this->getForm())
        throw new GhGuiException('The GhGuiDropDownButton has not been added to a GhGuiForm');
    }

    if(!empty($this->_keyboardShortcuts))
    {
      $page = $this->getPage();

      foreach($this->_keyboardShortcuts as $shortcut)
        $page->addKeyboardShortcut($shortcut[0], $shortcut[1], new GhGuiEventJavascript('function(event){' . $this->getJavascriptWidgetVariable() . '.onClick(event);}'));
    }

    return true;
  }
}