<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Radio-Button Komponente.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiRadioButton extends GhGuiFormComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'input';

  /**
   * Attribute für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_attributes = array('type' => 'radio');

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.form.RadioButton';

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onChange');

  /**
   * Konstruktor.
   *
   * Optional kann der Name des Eingabefeldes festgelegt werden (siehe
   * setName()).
   * Zusätzlich können der Wert (siehe setValue()) und der Vorgabestatus (siehe
   * setDefaultValue()) für das Formularfeld angegeben werden.
   *
   * @param string  $name           Name des Formularfeldes
   * @param string  $value          Wert für das Formularfeld
   * @param boolean $defaultChecked Vorgabewert für den Status des Radio-Buttons (true: markiert, false: leer)
   */
  public function __construct($name = null, $value = null, $defaultChecked = null)
  {
    parent::__construct($name, $value);

    if($defaultChecked !== null)
      $this->setDefaultValue($defaultChecked);
  }

  /**
   * Setzt den Status des Radio-Buttons.
   * Der Status true markiert das Radio-Button, der Status false leert es.
   *
   * @param boolean $checked Radio-Button Status
   * @return GhGuiCheckBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setChecked($checked)
  {
    $this->setWidgetProperty('checked', $checked ? true : false);

    return $this;
  }

  /**
   * Liefert den Status des Radio-Buttons.
   *
   * @return boolean
   */
  public function getChecked()
  {
    $checked = $this->getWidgetProperty('checked');

    if($checked !== null)
      return $checked;
    elseif($this->getUseCurrentValue() && $this->getName() && $this->getValue() !== null)
    {
      $value = GhCoreGlobals::getValue($this->getName());

      if($value !== null && $value == $this->getValue())
        return $value ? true : false;
    }

    return $this->getDefaultValue() ? true : false;
  }

  /**
   * Liefert die Objekteigenschaften für das Widget.
   *
   * @return array
   */
  protected function _getWidgetCreationProperties()
  {
    $properties = $this->getWidgetProperties();

    $properties['checked'] = $this->getChecked();

    return $properties;
  }
}