<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Textbereich Komponente (Text-Area).
 *
 * Einzeilige Textbereiche können durch die [classes]GhGuiTextBox[/classes]
 * Komponente dargestellt werden.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiTextArea extends GhGuiFormComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'textarea';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.form.SimpleTextarea';

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onChange');

  /**
   * Konstruktor.
   *
   * Optional kann der Name des Eingabefeldes (siehe setName()), der
   * Vorgabeinhalt (siehe setDefaultValue()), sowie die Größe des Textbereichs
   * in Zeilen und Spalten festgelegt werden.
   *
   * @param string  $name         Name des Formularfeldes
   * @param string  $defaultValue Vorgabeinhalt für das Formularfeld
   * @param int     $rows         Anzahl der Zeilen des Textbereichs
   * @param int     $columns      Anzahl der Spalten des Textbereichs
   */
  public function __construct($name = null, $defaultValue = null, $rows = null, $columns = null)
  {
    parent::__construct($name, null, $defaultValue);

    if($rows !== null)
      $this->setRows($rows);
    if($columns !== null)
      $this->setColumns($columns);
  }

  /**
   * Setzt die Anzahl der Zeilen des Textbereichs.
   *
   * @param int $rows Anzahl der Zeilen des Textbereichs
   * @return GhGuiTextArea liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setRows($rows)
  {
    $this->setWidgetProperty('rows', (int)$rows);

    return $this;
  }

  /**
   * Liefert die Anzahl der Zeilen des Textbereichs.
   *
   * @return int
   */
  public function getRows()
  {
    return $this->getWidgetProperty('rows');
  }

  /**
   * Setzt die Anzahl der Spalten des Textbereichs.
   *
   * @param int $columns Anzahl der Spalten des Textbereichs
   * @return GhGuiTextArea liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setColumns($columns)
  {
    $this->setWidgetProperty('cols', (int)$columns);

    return $this;
  }

  /**
   * Liefert die Anzahl der Spalten des Textbereichs.
   *
   * @return int
   */
  public function getColumns()
  {
    return $this->getWidgetProperty('cols');
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'rows':
      case 'cols':
        return 'int';
    }

    return parent::__getWidgetPropertyType($property);
  }
}