<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Baum Komponente.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiTree extends GhGuiFormComponent
{
  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.Tree';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dojo.data.ObjectStore', 'dojo.store.Memory', 'dijit.tree.ForestStoreModel');

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onClick');

  /**
   * Baum (Daten).
   * @var GhCoreTree
   */
  protected $_tree;

  /**
   * Eigenschaft der Datenobjekte für den Wert des Formularfeldes.
   * Die Vorgabeeigenschaft ist 'ID'.
   * @var string
   */
  protected $_valueProperty = 'ID';

  /**
   * ID des (versteckten) Formularfeldes, das den Wert der Auswahl aufnimmt.
   * @var string
   */
  protected $_hiddenFieldId;

  /**
   * Wert des vorausgewählten Baumknotens.
   * @var string
   */
  protected $_value;

  /**
   * Standard-Icon für (geschlossene) Knoten des Baums.
   * @var string
   */
  protected $_iconNode;

  /**
   * Standard-Icon für geöffnete Knoten des Baums.
   * @var string
   */
  protected $_iconNodeOpen;

  /**
   * Standard-Icon für Blätter des Baums.
   * @var string
   */
  protected $_iconLeaf;

  /**
   * Objekteigenschaft für Icons der Baumknoten.
   * @var string
   */
  protected $_iconProperty;

  /**
   * Objekteigenschaft für Icons der geöffneten Baumknoten.
   * @var string
   */
  protected $_iconOpenProperty;

  /**
   * Legt fest, ob die Attribute 'name' und 'value' für den HTML-Knoten erzeugt
   * werden.
   * @var boolean
   */
  protected $_createNameAndValueAttributes = false;

  /**
   * Legt fest, ob die Eigenschaften 'name' und 'value' für das Javascript
   * Widget erzeugt werden.
   * @var boolean
   */
  protected $_createNameAndValueWidgetProperties = false;

  /**
   * Zusätzliche Objekteigenschaften, die an den Baumelementen hinterlegt
   * werden.
   * @var array
   */
  protected $_additionalProperties;

  /**
   * Konstruktor.
   *
   * Optional können der Name des Formularfeldes und der Baum für die Auswahl
   * angegeben werden.
   * Zudem kann die Objekteigenschaft bzw. der Array-Schlüssel angegeben
   * werden, über die bei Auswahl eines Knotens aus den Daten der Baumknoten
   * der Wert für das Formularfeld bezogen wird.
   *
   * Wenn es sich bei den Daten in den Baumknoten um Objekte handelt, dann wird
   * der Wert für das Formularfeld aus der angegebenen Objekteigenschaft
   * bezogen wenn ein Baumknoten ausgewählt wird. Handelt es sich bei den Daten
   * um Arrays, so wird die Eigenschaft als Schlüssel verwendet.
   *
   * Wird false übergeben, so werden die Daten der Baumknoten direkt als Wert
   * für das Formularfeld verwendet, ohne sie als Objekt oder Array zu
   * interpretieren. Die Daten müssen in diesem Fall skalar sein (Strings oder
   * Zahlen).
   *
   * @param string     $name          Name des Formularfeldes
   * @param GhCoreTree $tree          Baum mit den Daten für den Drop-Down Baum
   * @param string     $valueProperty Objekteigenschaft
   */
  public function __construct($name = null, GhCoreTree $tree = null, $valueProperty = null)
  {
    parent::__construct();
    $this->_hiddenFieldId = GhGui::createId();

    if($name !== null)
      $this->setName($name);

    if($tree !== null)
      $this->setTree($tree);

    if($valueProperty !== null)
      $this->setValueProperty($valueProperty);
  }

  /**
   * Setzt den darzustellenden Baum.
   *
   * @param GhCoreTree $tree Baum
   * @return GhGuiTree liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTree(GhCoreTree $tree)
  {
    $this->_tree = $tree;

    return $this;
  }

  /**
   * Liefert den darzustellenden Baum.
   * 
   * @return GhCoreTree
   */
  public function getTree()
  {
    return $this->_tree;
  }

  /**
   * Serzt den Wert dieses Formularfeldes.
   *
   * @param string $value Wert für das Formularfeld
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValue($value)
  {
    // Keine Widget Eigenschaft verwenden:
    $this->_value = $value;

    return $this;
  }

  /**
   * Liefert den Wert des Formularfeldes.
   *
   * @return string
   */
  public function getValue()
  {
    return $this->_value;
  }

  /**
   * Liefert die ID des versteckten Formularfeldes, das den Wert der Auswahl
   * aufnimmt.
   *
   * @return string
   */
  public function getValueNodeId()
  {
    return $this->_hiddenFieldId;
  }

  /**
   * Setzt die Objekteigenschaft der Datenobjekte für den Wert des Formularfeldes.
   *
   * Wenn es sich bei den Daten in den Baumknoten um Objekte handelt, dann wird
   * der Wert für das Formularfeld aus der angegebenen Objekteigenschaft
   * bezogen wenn ein Baumknoten ausgewählt wird. Handelt es sich bei den Daten
   * um Arrays, so wird die Eigenschaft als Schlüssel verwendet.
   *
   * Wird false übergeben, so werden die Daten der Baumknoten direkt als Wert
   * für das Formularfeld verwendet, ohne sie als Objekt oder Array zu
   * interpretieren. Die Daten müssen in diesem Fall skalar sein (Strings oder
   * Zahlen).
   *
   * @param string $property Objekteigenschaft
   * @return GhGuiTree liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValueProperty($property)
  {
    $this->_valueProperty = $property;

    return $this;
  }

  /**
   * Liefert die Objekteigenschaft der Datenobjekte bzw. den Schlüssel der
   * Daten-Arrays für den Wert des Formularfeldes.
   *
   * @return string
   */
  public function getValueProperty()
  {
    return $this->_valueProperty;
  }

  /**
   * Setzt die Icons, die für Elemente des Baums verwendet werden.
   *
   * @param string $nodeIcon     Icon für (geschlossene) Baumknoten
   * @param string $openNodeIcon Icon für geöffnete Baumknoten
   * @param string $leafIcon     Icon für Blätter
   * @return GhGuiTree liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTreeIcons($nodeIcon, $openNodeIcon, $leafIcon)
  {
    if($nodeIcon !== null)
      $this->_iconNode = $nodeIcon;

    if($openNodeIcon !== null)
      $this->_iconNodeOpen = $openNodeIcon;

    if($leafIcon !== null)
      $this->_iconLeaf = $leafIcon;

    return $this;
  }

  /**
   * Setzt die Objekteigenschaften, die die Namen für die Icons der Elemente
   * des Baums enthalten.
   *
   * Die Daten der Baumobjekte müssen in den angegebenen
   * Eigenschaften/Schlüsseln die Namen der Icons aus dem Icon-Set enthalten.
   * Wird kein Icon für geöffnete Elemente angegeben, so wird dasselbe wie für
   * geschlossene Elemente verwendet.
   *
   * @param string $nodeIconProperty     Icon
   * @param string $openNodeIconProperty Icon für geöffnetes Element
   * @return GhGuiTree liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTreeIconProperties($nodeIconProperty, $openNodeIconProperty = null)
  {
    $this->_iconProperty = $nodeIconProperty;
    $this->_iconOpenProperty = $openNodeIconProperty;

    return $this;
  }

  /**
   * Legt weitere Objekteigenschaften fest, die an den Baumobjekten hinterlegt
   * werden.
   *
   * Dies kann genutzt werden, um weitere Daten für Javascript-Event-Handler
   * an den Baumelementen zu hinterlegen.
   *
   * @param string|array $properties     zusätzliche Eigenschaft (oder ein Array von Eigenschaften)
   * @param string       $properties,... weitere zusätzliche Eigenschaften
   * @return GhGuiTree liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setAdditionalProperties($properties)
  {
    if(!is_array($properties))
      $properties = array($properties);

    for($i = 1; $i < func_num_args(); $i++)
      $properties[] = func_get_arg($i);

    $this->_additionalProperties = $properties;

    return $this;
  }

  /**
   * Liefert zusätzliche Objekteigenschaften, die an den Baumobjekten
   * hinterlegt werden.
   *
   * @return array
   */
  public function getAdditionalProperties()
  {
    return is_array($this->_additionalProperties) ? $this->_additionalProperties : array();
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   *
   * Registriert ein verstecktes Formularfeld für den ausgewählten Baumknoten.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $name = $this->getName();

    if($name !== null)
    {
      $this->getForm()->addHiddenField($name, $this->getCurrentValue(), $this->getValueNodeId());
      $this->_initHtmlEvents();
    }

    return true;
  }

  /**
   * Registriert den Event-Handler für Events im Tree.
   */
  protected function _initHtmlEvents()
  {
    $this->addEventHandler('onClick', new GhGuiEventJavascript('function(item,node,event){dojo.byId("' . $this->getValueNodeId() . '").value=item.value;}'));
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    return GhGui::createHtml('div', array('id' => $this->getId()));
  }

  /**
   * Liefert den Javascript Code für diese Komponente.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    $javascript = 'var store_' . $this->getId() . '=new dojo.data.ObjectStore({objectStore:new dojo.store.Memory({data:[';

    if($this->_tree instanceof GhCoreTree)
      $javascript .= $this->_getTreeJson($this->_tree);

    $javascript .= ']})});';
    $javascript .= 'var model_' . $this->getId() . '=new dijit.tree.ForestStoreModel({store:store_' . $this->getId() . ',rootId:"' . GhGui::createId() . '",rootLabel:"#"});' . GhGui::$eol;

    $properties = $this->_getWidgetCreationProperties();
    $properties['model'] = 'model_' . $this->getId();
    $properties['showRoot'] = false;
    $properties['persist'] = false;

    // Icons:
    if($this->_iconProperty !== null || $this->_iconOpenProperty !== null || $this->_iconNode !== null || $this->_iconNodeOpen !== null || $this->_iconLeaf !== null)
    {
      $iconSet = $this->getIconSet(true);
      if($iconSet)
      {
        $iconClass = $iconSet->getIconSetClass();
        $iconsJavascript = 'function(item,opened){' . GhGui::$eol;

        // Icons in Abhängigkeit von Dateneigenschaften:
        if($this->_iconProperty !== null && $this->_iconOpenProperty !== null)
          $iconsJavascript .= 'if(item.iconOpen && opened)return "' . $iconClass . ' "+item.iconOpen;else if(item.icon)return "' . $iconClass . ' "+item.icon;' . GhGui::$eol;
        elseif($this->_iconProperty !== null && $this->_iconOpenProperty === null)
          $iconsJavascript .= 'if(item.icon)return "' . $iconClass . ' "+item.icon;' . GhGui::$eol;

        // Standard-Icons dieses Baums:
        if($this->_iconLeaf !== null)
          $iconLeaf = $iconClass . ' ' . $this->_iconLeaf;
        else
          $iconLeaf = 'dijitLeaf';

        if($this->_iconNode !== null)
          $iconNode = $iconClass . ' ' . $this->_iconNode;
        else
          $iconNode = 'dijitFolderClosed';

        if($this->_iconNodeOpen !== null)
          $iconNodeOpen = $iconClass . ' ' . $this->_iconNodeOpen;
        else
          $iconNodeOpen = 'dijitFolderOpened';

        $iconsJavascript .= 'if(!item.children || item.children.length==0)return "' . $iconLeaf . '";else if(opened)return "' . $iconNodeOpen . '";else return "' . $iconNode . '";' . GhGui::$eol;

        $iconsJavascript .= '}' . GhGui::$eol;
        $properties['getIconClass'] = $iconsJavascript;
      }
    }

    // Baum erzeugen:
    $javascript .= $this->_getWidgetCreationJavascript($this->_widgetClass, $properties);

    // Dem letzten Root-Knoten die Klasse 'ghGuiTreeLastRoot' geben, damit das Style korrekt gesetzt werden kann:
    $javascript .= 'if(' . $this->getJavascriptWidgetVariable() . '.rootNode.containerNode.children.length)dojo.addClass(' . $this->getJavascriptWidgetVariable() . '.rootNode.containerNode.children[' . $this->getJavascriptWidgetVariable() . '.rootNode.containerNode.children.length-1],"ghGuiTreeLastRoot");' . GhGui::$eol;

    // Vorausgewähltes Element selektieren:
    $value = $this->getCurrentValue();

    if($this->getName() !== null && $value !== null)
      $javascript .= 'var treePath=greyhound.dojo.getTreePath(' . $this->getJavascriptWidgetVariable() . ',"value","' . str_replace('"', '\\"', $value) . '");if(treePath)' . $this->getJavascriptWidgetVariable() . '.set("path",treePath);' . GhGui::$eol;

    return $javascript;
  }

  /**
   * Liefert den Javascript JSON Code für die Daten des Baumes.
   *
   * @param GhCoreTree $tree Baum
   * @return string
   */
  protected function _getTreeJson($tree)
  {
    $json = '';

    $iconProperty = $this->_iconProperty;
    $iconOpenProperty = $this->_iconOpenProperty;

    if($iconProperty !== null && $iconOpenProperty === null)
      $iconOpenProperty = $iconProperty;

    $first = true;

    foreach($tree->getChildren() as $treeNode)
    {
      if (!$first)
        $json .= ',';

      $first = false;

      $json .= GhGui::$eol . $this->_getTreeNodeJson($treeNode, $this->getValueProperty(), $iconProperty, $iconOpenProperty);
    }

    return $json;
  }

  /**
   * Liefert den Javascript JSON Code für die Daten eines Knotens im Baum.
   *
   * @param GhCoreTreeNode $treeNode         Baumknoten
   * @param string         $valueProperty    Dateneigenschaft für den Wert des Formularfelds
   * @param string         $iconProperty     Dateneigenschaft für das Icon des Baumknotens
   * @param string         $iconOpenProperty Dateneigenschaft für das Icon des geöffneten Baumknotens
   * @return string
   */
  protected function _getTreeNodeJson($treeNode, $valueProperty, $iconProperty, $iconOpenProperty)
  {
    $data = $treeNode->getData();

    if($valueProperty === false)
      $value = $data;
    elseif(is_object($data) && isset($data->$valueProperty) && is_scalar($data->$valueProperty))
      $value = $data->$valueProperty;
    elseif(is_array($data) && isset($data[$valueProperty]) && is_scalar($data[$valueProperty]))
      $value = $data[$valueProperty];
    else
      $value = '';

    $json = '{label:"' . GhCoreStrings::attributeValue($treeNode->getCaption())
            . '",value:"' . GhCoreStrings::attributeValue($value)
            . '",id:"' . GhGui::createId() . '"';

    // Zusätzliche Eigenschaften:
    $additionalProperties = $this->getAdditionalProperties();

    if(is_array($additionalProperties))
    {
      foreach($additionalProperties as $property)
      {
        if(is_object($data) && isset($data->$property))
          $json .= ',' . GhCoreStrings::jsonProperty($property, $data->$property);
        elseif(is_array($data) && isset($data[$property]))
          $json .= ',' . GhCoreStrings::jsonProperty($property, $data[$property]);
      }
    }

    // Icon:
    if($iconProperty !== null)
    {
      if(is_object($data) && isset($data->$iconProperty))
        $icon = (string)$data->$iconProperty;
      elseif(is_array($data) && isset($data[$iconProperty]))
        $icon = (string)$data[$iconProperty];
      else
        $icon = '';

      $json .= ',icon:"' . GhCoreStrings::attributeValue($icon) . '"';
    }

    // Icon für geöffneten Baumknoten:
    if($iconOpenProperty !== null)
    {
      if(is_object($data) && isset($data->$iconOpenProperty))
        $icon = (string)$data->$iconOpenProperty;
      elseif(is_array($data) && isset($data[$iconOpenProperty]))
        $icon = (string)$data[$iconOpenProperty];
      else
        $icon = '';

      $json .= ',iconOpen:"' . GhCoreStrings::attributeValue($icon) . '"';
    }

    // Kind-Elemente:
    $children = $treeNode->getChildren();
    if(!empty($children))
    {
      $json .= ',children:[';
      $first = true;

      foreach($children as $child)
      {
        if (!$first)
          $json .= ',';

        $first = false;

        $json .= $this->_getTreeNodeJson($child, $valueProperty, $iconProperty, $iconOpenProperty);
      }

      $json .= ']';
    }

    $json .= '}';

    return $json;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected  function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'showRoot':
      case 'persist':
      case 'autoExpand':
        return 'boolean';

      case 'model':
        return 'object';

      case 'getIconClass':
        return 'function';
    }

    return parent::__getWidgetPropertyType($property);
  }
}