<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Akkordeon-Container.
 *
 * Diese Strukturkomponente ordnet Komponenten in Karten untereinander an,
 * von denen immer nur eine sichtbar ist. Jede Karte besitzt eine Titelleiste,
 * die beim Anklicken die vorher sichtbare Komponente verbirgt und die
 * angeklickte öffnet.
 * 
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiAccordionContainer extends GhGuiContainer
{
  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.AccordionContainer';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dijit.layout.ContentPane');

  /**
   * Eigenschaften für das aus dieser Komponente erzeugte Widget Objekt.
   * @var array
   */
  protected $_widgetProperties = array('persist' => false);

  /**
   * Karten.
   * Die Schlüssel sind die Beschriftungen und die Werte die Komponenten.
   * @var array
   */
  protected $_cards = array();

  /**
   * Aktive Karte.
   * Enthält diese Eigenschaft den Titel (die Beschriftung) einer
   * Karte, so wird diese Karte bei der Darstellung des Akkordeons
   * vorausgewählt.
   * @var string
   */
  protected $_selectedCard;

  /**
   * Fügt eine Karte hinzu.
   *
   * @param string         $caption   Beschriftung
   * @param GhGuiComponent $component Komponente
   * @param string         $icon      Name des Icons (aus dem Icon-Set)
   * @return GhGuiComponent liefert die zur neuen Karte hinzugefügte Komponente zurück
   */
  public function addCard($caption, GhGuiComponent $component, $icon = null)
  {
    // Inhalte werden immer in eine ContentPane gesetzt, diese wird beim Ändern der Größe auf die Content-Box
    // des jeweiligen AccordionContainers angepasst (Workaround gegen ungewollte Scrollbars beim Verkleinern von Akkordeon-Karten).

    $card = new GhGuiPanel($component);
    $card->setWidgetProperty('title', $caption);
    $card->addClass('GhGuiIsSingleChild');

    if($icon !== null)
      $card->setIcon($icon);

    $card->_setParentContainer($this);
    $this->_cards[] = $card;

    return $component;
  }

  /**
   * Setzt die aktive Karte.
   *
   * Es wird der Titel (die Beschriftung) der Karte übergeben, die
   * vorausgewählt (aktiv dargestellt) werden soll.
   *
   * @param string $caption Beschriftung der Karte
   * @return GhGuiTabContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setSelectedCard($caption)
  {
    $this->_selectedCard = $caption;

    return $this;
  }

  /**
   * Liefert die aktive Karte.
   *
   * Falls keine aktive Karte gewählt wurde (null) wird die erste Karte
   * aktiviert.
   *
   * @return string
   */
  public function getSelectedCard()
  {
    return $this->_selectedCard;
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  protected function _getAllComponents()
  {
    return array_values($this->_cards);
  }

  /**
   * Initialisiert die Komponente und alle in ihr enthaltenen Komponenten für
   * das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $selectedCard = $this->getSelectedCard();
    if($selectedCard !== null)
    {
      foreach($this->_getAllComponents() as $component)
      {
        $caption = $component->getWidgetProperty('title');
        if($caption == $selectedCard)
        {
          $component->setWidgetProperty('selected', true);
          break;
        }
      }
    }

    return true;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'persist':
        return 'boolean';
    }

    return parent::__getWidgetPropertyType($property);
  }

  /**
   * Liefert Javascript Code für diese Komponente, der nach dem Erzeugen der
   * Komponenten ausgeführt wird.
   *
   * Dieser Code wird in dojo.ready() ausgeführt, nachdem der Javascript-Code
   * den die Komponenten mit _getJavascript() liefern, ausgeführt wird.
   *
   * @return string
   */
  protected function _getJavascriptAfterComponentCreation()
  {
    $javascript = parent::_getJavascriptAfterComponentCreation();

    // Erneutes Layout erzwingen, da sonst Komponenten mit fillWidth oder
    // fillHeight nicht korrekt verarbeitet werden (es entsteht unnötiger
    // Platz):
    $javascript .= $this->getJavascriptWidgetVariable() . '.resize();';

    return $javascript;
  }
}