<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Modaler Dialog.
 *
 * Wird ein Dialog zur Webseite hinzugefügt, so überlagert er alle anderen
 * Inhalte und blockiert diese (der Dialog ist modal). Nur der Dialog kann
 * in diesem Fall abgesendet werden.
 *
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiDialog extends GhGuiContainer
{
  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.Dialog';

  /**
   * Dialogtitel/Beschriftung.
   * @var string
   */
  protected $_caption;

  /**
   * Formular.
   * @var GhGuiForm
   */
  protected $_form;

  /**
   * Container für die Inhalte des Dialogs.
   * @var GhGuiPanel
   */
  protected $_contentContainer;

  /**
   * Container für Buttons um den Dialog zu schließen bzw. abzuschicken.
   * @var GhGuiPanel
   */
  protected $_actionContainer;

  /**
   * Einstellung, ob der Dialog beim Laden der Seite automatisch angezeigt
   * werden soll.
   * @var boolean
   */
  protected $_showDialog = true;

  /**
   * Konstruktor.
   *
   * Optional kann die Beschriftung (der Titel) des Dialogs, sowie die Breite
   * und Höhe (als CSS Angabe) gesetzt werden.
   *
   * Die Größenangabe bezieht sich auf die Größe des Dialogs. Für die Inhalte
   * steht entsprechend weniger Platz zur Verfügung, da der Dialogtitel und die
   * Schaltflächen-Zeile in der Höhe inbegriffen sind. Prozentuale Angaben
   * beziehen sich auf die Größe der Webseite. Wird keine Höhe angegeben, so
   * stellt sich der Dialog so groß dar, dass alle Inhalte hinein passen.
   *
   * Der Dialog enthält ein Formular, welches zum Schließen des Dialogs
   * abgeschickt wird. Die Formularaktion, Request-Methode und das Ziel des
   * Formulars können ebenfalls optional angegeben werden
   * (siehe [classes]GhGuiForm[/classes]).
   *
   * @param string $caption    Beschriftung (Dialogtitel)
   * @param string $width      Breite des Dialogs (Pixelangabe)
   * @param string $height     Höhe des Dialogs (Pixelangabe)
   * @param string $action     Formularaktion
   * @param string $method     Request-Methode
   * @param string|GhGuiIframe $target Ziel des Formulars
   */
  public function __construct($caption = null, $width = null, $height = null, $action = null, $method = null, $target = null)
  {
    parent::__construct();
    $this->setWidgetProperty('closable', false);

    $this->_form = new GhGuiForm($action, $method, $target);
    $this->_form->_setParentContainer($this);

    $this->_contentContainer = new GhGuiPanel();
    $this->_contentContainer->addClass('dijitDialogPaneContentArea');
    $this->_contentContainer->setStyle('padding', '8px');
    $this->_form->addComponent($this->_contentContainer);
    $actionWrapper = new GhGuiPanel();
    $this->_form->addComponent($actionWrapper);
    $actionWrapper->addClass('dijitDialogPaneActionBar');
    $this->_actionContainer = new GhGuiPanel();
    $this->_actionContainer->setStyle('padding', '8px');
    $actionWrapper->addComponent($this->_actionContainer);

    if($caption !== null)
      $this->setCaption($caption);

    if($width !== null)
      $this->setStyle('width', $width);

    if($height !== null)
      $this->setStyle('height', $height);
  }

  /**
   * Setzt die Beschriftung (den Titel) des Dialogs.
   *
   * @param string $caption Beschriftung (Dialogtitel)
   * @return GhGuiDialog liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCaption($caption)
  {
    $this->setWidgetProperty('title', $caption);

    return $this;
  }

  /**
   * Liefert die Beschriftung (den Dialogtitel).
   *
   * @return string
   */
  public function getCaption()
  {
    return $this->getWidgetProperty('title');
  }

  /**
   * Liefert das Formular dieses Dialogs.
   *
   * @return GhGuiForm
   */
  public function getForm()
  {
    return $this->_form;
  }

  /**
   * Liefert den Container, in dem die Inhaltskomponenten des Dialogs
   * enthalten sind.
   *
   * @return GhGuiPanel
   */
  public function getContentContainer()
  {
    return $this->_contentContainer;
  }

  /**
   * Liefert den Container, in dem die Buttons des Dialogs enthalten sind.
   *
   * @return GhGuiPanel
   */
  public function getButtonsContainer()
  {
    return $this->_actionContainer;
  }

  /**
   * Fügt eine Komponente zum Inhaltsbereich des Dialogs hinzu.
   *
   * @param GhGuiComponent $component Komponente
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addContent(GhGuiComponent $component)
  {
    return $this->_contentContainer->addComponent($component);
  }

  /**
   * Fügt eine Komponente zum Buttons-Bereich des Dialogs hinzu.
   *
   * Üblicherweise werden nur Buttons zu diesem Bereich hinzugefügt, es ist
   * aber möglich, beliebige Komponenten hinzuzufügen.
   *
   * @param GhGuiComponent $component Komponente
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addButton(GhGuiComponent $component)
  {
    return $this->_actionContainer->addComponent($component);
  }

  /**
   * Setzt eine CSS Eigenschaft dieser Komponente.
   *
   * @param string $style CSS Eigenschaft
   * @param string $value Wert der CSS Eigenschaft (null: die Eigenschaft wird entfernt)
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setStyle($style, $value)
  {
    switch(strtolower($style))
    {
      case 'height':
        if(is_numeric($value))
          $value .= 'px';

        // Am Widget vermerken, ob die Höhe vorgegeben ist oder nicht:
        if($value)
          $this->setWidgetProperty('ghHeight', $value);
        else
          $this->setWidgetProperty('ghHeight', null);
        break;

      case 'width':
        if(is_numeric($value))
          $value .= 'px';
        break;
    }

    return parent::setStyle($style, $value);
  }

  /**
   * Legt fest, ob der Dialog beim Laden der Seite automatisch angezeigt wird.
   *
   * @param boolean $show true: Dialog wird angezeigt, false: Dialog wird nicht
   *   angezeigt
   * @return GhGuiDialog liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setShow($show)
  {
    $this->_showDialog = $show ? true : false;

    return $this;
  }

  /**
   * Liefert die Einstellung, ob der Dialog beim Laden der Seite angezeigt
   * werden soll.
   *
   * @return boolean
   */
  public function getShow()
  {
    return $this->_showDialog;
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  protected function _getAllComponents()
  {
    // Das Formular enthält den Content- und Action-Bereich:
    return array($this->_form);
  }

  /**
   * Liefert den Javascript Code für diese Komponente und die darin enthaltenen
   * Komponenten.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    $javascript = parent::_getJavascript();

    $javascript .= $this->getJavascriptWidgetVariable() . '.ghDialogContent=' . $this->_contentContainer->getJavascriptWidgetVariable() . ';' . GhGui::$eol;
    $javascript .= $this->getJavascriptWidgetVariable() . '.ghDialogActionBar=' . $this->_actionContainer->getJavascriptWidgetVariable() . ';' . GhGui::$eol;

    // Dialog anzeigen:
    if($this->getShow())
      $javascript .= '' . $this->getJavascriptWidgetVariable() . '.show();';

    return $javascript;
  }
}
