<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Formular (HTML 'form' Knoten).
 *
 * Dieser Container erzeugt ein HTML Formular ('form' Knoten). Komponenten,
 * die von GhGuiFormComponent abgeleitet sind und in dem Container enthalten
 * sind, übermitteln ihre Werte beim Absenden des Formulars.
 * Formularkomponenten, deren auto-submit Funktion aktiviert ist, senden das
 * Formular beim Ändern ihres Wertes automatisch ab.
 *
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiForm extends GhGuiPanel
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'form';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.form.Form';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dijit.layout.ContentPane');

  /**
   * CSS Angaben für den Haupt-DOM-Knoten dieser Komponente.
   * Formulare erhalten 100% Breite und Höhe.
   * @var array
   */
  protected $_styles = array('width' => '100%', 'height' => '100%');

  /**
   * Namen für versteckte Formularfelder (hidden fields).
   * @var array
   */
  protected $_hiddenFields = array();

  /**
   * Container, in dem die versteckten Formularfelder dieses Formulars
   * enthalten sind.
   * @var GhGuiPanel
   */
  protected $_hiddenFieldsContainer;

  /**
   * Ziel des Formulars.
   * Dies kann ein 'target' String, oder eine GhGuiIframe Komponente sein
   * (deren ID als 'target' String verwendet wird).
   * @var string|GhGuiIframe
   */
  protected $_target;

  /**
   * Konstruktor.
   *
   * Optional kann die Formularaktion (siehe setAction()), die Request-Methode
   * (siehe setMethod()) und das Ziel des Formulars (siehe setTarget())
   * angegeben werden.
   *
   * @param string             $action Formularaktion
   * @param string             $method Request-Methode
   * @param string|GhGuiIframe $target Ziel des Formulars
   */
  public function __construct($action = null, $method = null, $target = null)
  {
    parent::__construct();

    $this->_hiddenFieldsContainer = new GhGuiPanel();
    $this->_hiddenFieldsContainer->setStyle('display', 'none');
    $this->_hiddenFieldsContainer->setStyle('width', '0px');
    $this->_hiddenFieldsContainer->setStyle('height', '0px');
    $this->addComponent($this->_hiddenFieldsContainer);

    if($action !== null)
      $this->setAction($action);

    if($method === null)
      $method = 'post';
    $this->setMethod($method);

    if($target !== null)
      $this->setTarget($target);
  }

  /**
   * Setzt die Aktion des Formulars (das 'action' Attribut).
   *
   * Dies ist in der Regel die URL eines Scriptes, welches die abgeschickten
   * Daten entgegen nimmt. Ist der Wert leer, so sendet das Formular die
   * Daten an das aktuell ausgeführte Script.
   *
   * @param string $action Formularaktion
   * @return GhGuiForm liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setAction($action)
  {
    $this->setWidgetProperty('action', $action, 'string');

    return $this;
  }

  /**
   * Liefert die Aktion des Formulars (das 'action' Attribut).
   *
   * @return string
   */
  public function getAction()
  {
    return $this->getWidgetProperty('action');
  }

  /**
   * Setzt die Request-Methode des Formulars ('post' oder 'get').
   *
   * @param string $method Request-Methode
   * @return GhGuiForm liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setMethod($method)
  {
    $this->setWidgetProperty('method', $method);

    return $this;
  }

  /**
   * Liefert die Request-Methode des Formulars.
   *
   * @return string
   */
  public function getMethod()
  {
    return $this->getWidgetProperty('method');
  }

  /**
   * Setzt den MIME-Typ des Formulars.
   *
   * @param string $encodingType MIME-Typ des Formulars
   * @return GhGuiForm liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setEncodingType($encodingType)
  {
    $this->setWidgetProperty('encType', $encodingType);
    $this->setAttribute('enctype', $encodingType);

    return $this;
  }

  /**
   * Liefert den MIME-Typ des Formulars.
   *
   * @return string
   */
  public function getEncodingType()
  {
    return $this->getWidgetProperty('encType');
  }

  /**
   * Fügt ein verstecktes Formularfeld (hidden field) zum Formular hinzu.
   *
   * @param string $name  Name des versteckten Formularfeldes
   * @param string $value Wert des versteckten Formularfeldes
   * @param string $id    ID des versteckten Formularfeldes
   * $param string $html  HTML Code für das versteckte Formularfeld
   * @return GhGuiHiddenField liefert das hinzugefügte versteckte Formularfeld zurück
   */
  public function addHiddenField($name, $value, $id = null, $html = null)
  {
    $component = new GhGuiHiddenField($name, $value, $id, $html);
    $this->_hiddenFieldsContainer->addComponent($component);

    return $component;
  }

  /**
   * Liefert die versteckten Formularfelder (hidden fields) des Formulars.
   *
   * @return array
   */
  public function getHiddenFields()
  {
    return $this->_hiddenFieldsContainer->getComponents();
  }

  /**
   * Liefert ein verstecktes Formularfeld anhand seines Namens.
   *
   * @param string $name Formularfeldname
   * @return GhGuiHiddenField
   */
  public function getHiddenField($name)
  {
    foreach($this->_hiddenFieldsContainer->getComponents() as $component)
      if($component->getName() == $name)
        return $component;

    return null;
  }

  /**
   * Setzt das Ziel des Formulars.
   *
   * Falls ein String als Ziel angegeben wird, erhält das Formular diesen als
   * 'target' Attribut, sendet also die Formulardaten in ein Fenster oder
   * IFrame mit diesem Namen. Falls eine GhGuiIFrame Komponente angegeben
   * wird, so wird deren ID als 'target' Attribut eingetragen, so dass das
   * Formular in dieses IFrame abgesendet wird.
   *
   * @param string|GhGuiIframe $target Ziel des Formulars
   * @return GhGuiForm liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTarget($target)
  {
    $this->_target = $target;

    return $this;
  }

  /**
   * Liefert das Ziel des Formulars.
   *
   * @return string|GhGuiIframe
   */
  public function getTarget()
  {
    return $this->_target;
  }

  /**
   * Liefert den Container, in dem die versteckten Formularfelder dieses
   * Formulars enthalten sind.
   *
   * @return GhGuiPanel
   */
  public function getHiddenFieldsContainer()
  {
    return $this->_hiddenFieldsContainer;
  }

  /**
   * Initialisiert die Komponente und alle in ihr enthaltenen Komponenten für
   * das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    // Ein Forumlar darf nicht innerhalb eines anderen Formulars eingefügt werden:
    $parent = $this->getParentContainer();
    while($parent)
    {
      if($parent instanceof GhGuiForm)
        throw new GhGuiException('GhGuiForm cannot be inserted into another GhGuiForm');
      $parent = $parent->getParentContainer();
    }

    // Target setzen:
    $target = $this->getTarget();
    if($target instanceof GhGuiIframe)
      $this->setWidgetProperty('target', $target->getId());
    elseif(!empty($target))
      $this->setWidgetProperty('target', $target);

    return true;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'action':
      case 'method':
      case 'target':
        return 'string';
    }

    return parent::__getWidgetPropertyType($property);
  }
}