<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Tab-Container (mit Karteireitern).
 *
 * Dieser Container stellt mehrere alternative Komponenten dar, zwischen denen
 * durch Auswahl eines Tabs (Karteireiters) gewechselt werden kann.
 *
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiTabContainer extends GhGuiContainer
{
  /**
   * Horizontale Karteireiter am oberen Rand.
   * @var const
   */
  const POSITION_TOP_HORIZONTAL = 'top-h';

  /**
   * Vertikale Karteireiter am oberen Rand.
   * @var const
   */
  const POSITION_TOP_VERTICAL = 'top-v';

  /**
   * Horizontale Karteireiter am rechten Rand.
   * @var const
   */
  const POSITION_RIGHT_HORIZONTAL = 'right-h';

  /**
   * Vertikale Karteireiter am rechten Rand.
   * @var const
   */
  const POSITION_RIGHT_VERTICAL = 'right-v';

  /**
   * Horizontale Karteireiter am unteren Rand.
   * @var const
   */
  const POSITION_BOTTOM_HORIZONTAL = 'bottom-h';

  /**
   * Vertikale Karteireiter am unteren Rand.
   * @var const
   */
  const POSITION_BOTTOM_VERTICAL = 'bottom-v';

  /**
   * Horizontale Karteireiter am linken Rand.
   * @var const
   */
  const POSITION_LEFT_HORIZONTAL = 'left-h';

  /**
   * Vertikale Karteireiter am linken Rand.
   * @var const
   */
  const POSITION_LEFT_VERTICAL = 'left-v';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.layout.TabContainer';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dijit.layout.ContentPane');

  /**
   * Tabs (Karteireiter).
   * Die Schlüssel sind die Beschriftungen und die Werte die Komponenten.
   * @var array
   */
  protected $_tabs = array();

  /**
   * Aktiver Karteireiter.
   * Enthält diese Eigenschaft den Titel (die Beschriftung) eines
   * Karteireiters, so wird dieser Karteireiter bei der Darstellung des
   * Tab-Containers vorausgewählt.
   * @var string
   */
  protected $_selectedTab;

  /**
   * Konstruktor.
   *
   * Optional kann die Position der Karteireiter angegeben werden:
   * <dl>
   * <dt><tt>GhGuiTabContainer::POSITION_TOP_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am oberen Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_TOP_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am oberen Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_RIGHT_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am rechten Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_RIGHT_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am rechten Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_BOTTOM_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am unteren Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_BOTTOM_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am unteren Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_LEFT_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am linken Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_LEFT_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am linken Rand.</dd>
   * </dl>
   *
   * @param string $tabPosition Karteireiterposition
   */
  public function __construct($tabPosition = null)
  {
    parent::__construct();
    $this->setWidgetProperty('controllerWidget', 'dijit.layout.ScrollingTabController');

    if($tabPosition === null)
      $tabPosition = self::POSITION_TOP_HORIZONTAL;
    $this->setTabPosition($tabPosition);
  }

  /**
   * Fügt einen Karteireiter (Tab) hinzu.
   *
   * @param string         $caption   Beschriftung
   * @param GhGuiComponent $component Komponente
   * @param boolean        $closable  true: Karteireiter ist schließbar, false:
   *   Karteireiter kann nicht geschlossen werden
   * @param string         $icon      Name des Icons (aus dem Icon-Set)
   * @return GhGuiComponent liefert die zum neuen Tab hinzugefügte Komponente zurück
   */
  public function addTab($caption, GhGuiComponent $component, $closable = false, $icon = null)
  {
    if($component->getWidgetProperty('title') !== null)
      $tab = new GhGuiPanel($component);
    else
      $tab = $component;

    $tab->setWidgetProperty('title', $caption);
    $tab->addClass('GhGuiIsSingleChild');
    $tab->setStyle('width', '100%');
    $tab->setStyle('height', '100%');
    
    if($closable)
      $tab->setWidgetProperty('closable', true);

    if($icon !== null)
      $tab->setIcon($icon);

    $tab->_setParentContainer($this);
    $this->_tabs[] = $tab;

    return $component;
  }

  /**
   * Legt die Position der Karteireiter am Container fest.
   *
   * <dl>
   * <dt><tt>GhGuiTabContainer::POSITION_TOP_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am oberen Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_TOP_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am oberen Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_RIGHT_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am rechten Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_RIGHT_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am rechten Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_BOTTOM_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am unteren Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_BOTTOM_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am unteren Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_LEFT_HORIZONTAL</tt></dt>
   * <dd>Die Karteireiter erscheinen horirontal am linken Rand.</dd>
   * <dt><tt>GhGuiTabContainer::POSITION_LEFT_VERTICAL</tt></dt>
   * <dd>Die Karteireiter erscheinen vertikal am linken Rand.</dd>
   * </dl>
   *
   * @param string $position Karteireiterposition
   * @return GhGuiTabContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTabPosition($position)
  {
    $this->setWidgetProperty('tabPosition', $position);

    return $this;
  }

  /**
   * Liefert die Position der Karteireiter am Container.
   *
   * @return string
   */
  public function getTabPosition()
  {
    return $this->getWidgetProperty('tabPosition');
  }

  /**
   * Setzt den aktiven Karteireiter.
   *
   * Es wird der Titel (die Beschriftung) des Karteireiters übergeben, der
   * vorausgewählt (aktiv dargestellt) werden soll.
   *
   * @param string $caption Beschriftung des Karteireiters
   * @return GhGuiTabContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setSelectedTab($caption)
  {
    $this->_selectedTab = $caption;

    return $this;
  }

  /**
   * Liefert den aktiven Karteireiter.
   *
   * Falls kein aktiver Karteireiter gewählt wurde (null) wird der erste
   * Karteireiter aktiviert.
   *
   * @return string
   */
  public function getSelectedTab()
  {
    return $this->_selectedTab;
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  protected function _getAllComponents()
  {
    return array_values($this->_tabs);
  }

  /**
   * Initialisiert die Komponente und alle in ihr enthaltenen Komponenten für
   * das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $selectedTab = $this->getSelectedTab();
    if($selectedTab !== null)
    {
      foreach($this->_getAllComponents() as $component)
      {
        $caption = $component->getWidgetProperty('title');
        if($caption == $selectedTab)
        {
          $component->setWidgetProperty('selected', true);
          break;
        }
      }
    }

    return true;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'controllerWidget':
      case 'tabPosition':
        return 'string';
    }

    return parent::__getWidgetPropertyType($property);
  }
}