<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Tooltip-Komponente.
 *
 * Diese Komponente stellt beliebige andere Komponenten in einem Tooltip dar.
 * Der Tooltip kann an Komponenten gebunden werden und wird neben diesen
 * angezeigt wenn der Mauszeiger über die Komponente fährt.
 * 
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiTooltip extends GhGuiPanel
{
  /**
   * Zeigt den Tooltip über der entsprechenden Komponente an.
   * @var const
   */
  const POSITION_TOP = 'above';

  /**
   * Zeigt den Tooltip rechts neben der entsprechenden Komponente an.
   * @var const
   */
  const POSITION_RIGHT = 'after';

  /**
   * Zeigt den Tooltip unter der entsprechenden Komponente an.
   * @var const
   */
  const POSITION_BOTTOM = 'below';

  /**
   * Zeigt den Tooltip links neben der entsprechenden Komponente an.
   * @var const
   */
  const POSITION_LEFT = 'before';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.Tooltip';

  /**
   * Konstruktor.
   *
   * Optional kann die Position des Tooltips in Relation zu der Komponente, an
   * der er angezeigt werden soll, festgelegt werden, sowie ein oder mehrere
   * Komponenten angegeben werden, die als Inhalte zum Tooltip hinzugefügt
   * werden.
   *
   * Folgende Positionen sind möglich:
   * <dl>
   * <dt>GhGuiTooltip::POSITION_TOP</dt>
   * <dd>Zeigt den Tooltip über der entsprechenden Komponente an.</dd>
   * <dt>GhGuiTooltip::POSITION_RIGHT</dt>
   * <dd>Zeigt den Tooltip rechts neben der entsprechenden Komponente an.</dd>
   * <dt>GhGuiTooltip::POSITION_BOTTOM</dt>
   * <dd>Zeigt den Tooltip unter der entsprechenden Komponente an.</dd>
   * <dt>GhGuiTooltip::POSITION_LEFT</dt>
   * <dd>Zeigt den Tooltip links neben der entsprechenden Komponente an.</dd>
   * </dl>
   * Die Voreinstellung ist GhGuiTooltip::POSITION_TOP.
   *
   * @param string         $position     Position des Tooltips
   * @param GhGuiComponent $component    Komponente
   * @param GhGuiComponent $component... weitere Komponenten
   */
  public function __construct($position = null, $component = null)
  {
    if(func_num_args() > 0)
    {
      $components = func_get_args();
      array_shift($components);
      call_user_func_array(array('parent', '__construct'), $components);
    }
    else
      parent::__construct();

    if($position === null)
      $position = self::POSITION_TOP;

    $this->setTooltipPosition($position);
  }

  /**
   * Legt die Position des Tooltips in Relation zu der Komponente, an der er
   * angezeigt werden soll, fest.
   *
   * Folgende Positionen sind möglich:
   * <dl>
   * <dt>GhGuiTooltip::POSITION_TOP</dt>
   * <dd>Zeigt den Tooltip über der entsprechenden Komponente an.</dd>
   * <dt>GhGuiTooltip::POSITION_RIGHT</dt>
   * <dd>Zeigt den Tooltip rechts neben der entsprechenden Komponente an.</dd>
   * <dt>GhGuiTooltip::POSITION_BOTTOM</dt>
   * <dd>Zeigt den Tooltip unter der entsprechenden Komponente an.</dd>
   * <dt>GhGuiTooltip::POSITION_LEFT</dt>
   * <dd>Zeigt den Tooltip links neben der entsprechenden Komponente an.</dd>
   * </dl>
   *
   * @param string $position Position des Tooltips
   * @return GhGuiTooltip liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTooltipPosition($position)
  {
    if($position)
      $this->setWidgetProperty('position', array($position));
    else
      $this->setWidgetProperty('position', null);

    return $this;
  }

  /**
   * Liefert die Position des Tooltips in Relation zu der Komponente, an der er
   * angezeigt werden soll.
   *
   * @return string
   */
  public function getTooltipPosition()
  {
    $position = $this->getWidgetProperty('position');

    if(!is_array($position) || empty($position))
      return null;

    return $position[0];
  }

  /**
   * Setzt die Verzögerung (in Millisekunden) bis zur Darstellung des Tooltips.
   *
   * @param int $delay Verzögerung in Millisekunden
   * @return GhGuiTooltip liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setDelay($delay)
  {
    $this->setWidgetProperty('showDelay', $delay);

    return $this;
  }

  /**
   * Liefert die Verzögerung (in Millisekunden) bis zur Darstellung des
   * Tooltips.
   *
   * @return int
   */
  public function getDelay()
  {
    return $this->getWidgetProperty('showDelay');
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected  function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'connectId':
      case 'position':
        return 'array';

      case 'showDelay':
        return 'int';
    }

    return parent::__getWidgetPropertyType($property);
  }
}