<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Basisklasse für GUI Komponenten.
 *
 * @package GhGui
 * @subpackage core
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
abstract class GhGuiComponent
{
  /**
   * ID der Komponente.
   * Diese ID wird im DOM Baum verwendet, um diese Komponente zu
   * identifizieren.
   * @var string
   */
  protected $_id;

  /**
   * Der Container, in dem diese Komponente enthalten ist.
   * @var GhGuiContainer
   */
  protected $_parentContainer;

  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'div';

  /**
   * Attribute für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_attributes = array();

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array();

  /**
   * CSS Eigenschaften für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_styles = array();

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass;

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array();

  /**
   * Eigenschaften für das aus dieser Komponente erzeugte Widget Objekt.
   * @var array
   */
  protected $_widgetProperties = array();

  /**
   * Event-Handler.
   * Die Schlüssel des Arrays sind Event-Namen, die Werte sind Arrays mit
   * Event-Handlern (Objekte vom Typ GhGuiEvent).
   * @var array
   */
  protected $_eventHandlers = array();

  /**
   * Status der HTML-Initialisierung.
   * Der Wert true bedeutet, dass _initHtml() bereits durchgeführt wurde,
   * false, dass _initHtml() noch nicht durchgeführt wurde.
   * @var boolean
   */
  protected $_initHtmlDone = false;

  /**
   * Icon-Set für diese Komponente.
   * @var GhGuiIconSet
   */
  protected $_iconSet;

  /**
   * Icon dieser Komponente.
   * Dieser Wert entspricht dem Namen eines Icons aus dem Icon-Set.
   * @var string
   */
  protected $_icon;

  /**
   * Tooltip der an diese Komponente gebunden ist.
   * @var array
   */
  protected $_tooltip;

  /**
   * Event, bei dem der Tooltip-Dialog angezeigt wird.
   * @var string
   */
  protected $_tooltipEvent;

  /**
   * Legt fest, ob die Komponente die restliche freie Breite einnimmt.
   * @var boolean
   */
  protected $_fillWidth;

  /**
   * Legt fest, ob die Komponente die restliche freie Höhe einnimmt.
   * @var boolean
   */
  protected $_fillHeight;

  /**
   * Trace data to identify the code position where this component was created.
   * This value is only filled when the debug mode has been activated.
   * @var array
   */
  protected $_creationTrace;

  /**
   * Konstruktor.
   *
   * Setzt die ID der Komponente auf einen eindeutigen Wert.
   */
  public function __construct()
  {
    if(GhGui::getDebugMode())
    {
      $trace = debug_backtrace(false);

      if(!empty($trace) && isset($trace[1]))
        $this->_creationTrace = $trace[1];
    }

    $this->setId(GhGui::createId());
  }

  /**
   * Liefert die ID dieser Komponente.
   *
   * @return string
   */
  public function getId()
  {
    return $this->_id;
  }

  /**
   * Setzt die ID dieser Komponente.
   *
   * Diese ID muss eindeutig sein und wird als ID für den Haupt-DOM-Knoten
   * dieser Komponente verwendet.
   *
   * @param string $id dom node id
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setId($id)
  {
    $this->_id = $id;

    return $this;
  }

  /**
   * Liefert den Container, in dem diese Komponente enthalten ist.
   *
   * Falls die Komponente in keinen Container eingefügt wurde wird null
   * zurück geliefert.
   *
   * @return GhGuiContainer
   */
  public function getParentContainer()
  {
    return $this->_parentContainer;
  }

  /**
   * Setzt den Container, in dem diese Komponente enthalten ist.
   *
   * Diese Methode wird von einem Container aufgerufen wenn eine Komponente
   * in ihn eingefügt wird.
   *
   * @param GhGuiContainer $container der Container, der diese Komponente enthält
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  protected function _setParentContainer(GhGuiContainer $container = null)
  {
    $this->_parentContainer = $container;

    return $this;
  }

  /**
   * Liefert die Webseitenkomponente, der diese Komponente angehört.
   *
   * @return GhGuiPage
   */
  public function getPage()
  {
    $parent = $this->getParentContainer();
    while($parent)
    {
      if($parent instanceof GhGuiPage)
        return $parent;
      $parent = $parent->getParentContainer();
    }

    return null;
  }

  /**
   * Setzt ein Attribut dieser Komponente.
   *
   * Die Attribute 'class' und 'style' werden ausschließlich über die Methoden
   * addClass() bzw. setStyle() gesetzt.
   *
   * @param string $attribute Attributsname
   * @param string $value     Wert des Attributs
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setAttribute($attribute, $value)
  {
    $this->_attributes[$attribute] = $value;

    return $this;
  }

  /**
   * Setzt die Attribute dieser Komponente.
   *
   * Die Attribute werden als DOM Attribute an den Haupt-DOM-Knoten dieser
   * Komponente gesetzt.
   *
   * @param array $attributes Attribute (Schlüssel-Wert Paare)
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setAttributes(array $attributes)
  {
    foreach($attributes as $attribute => $value)
      $this->setAttribute($attribute, $value);

    return $this;
  }

  /**
   * Entfernt ein Attribut von dieser Komponente.
   *
   * @param string $attribute Attribut
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function removeAttribute($attribute)
  {
    if(isset($this->_attributes[$attribute]))
      unset($this->_attributes[$attribute]);

    return $this;
  }

  /**
   * Liefert ein Attribut dieser Komponente.
   *
   * Die Attribute 'class' und 'style' werden ausschließlich über die Methoden
   * getClasses() bzw. getStyle() abgerufen.
   *
   * @param string $attribute Attributsname
   * @return string
   */
  public function getAttribute($attribute)
  {
    if (isset($this->_attributes[$attribute]))
      return $this->_attributes[$attribute];

    return null;
  }

  /**
   * Liefert die Attribute dieser Komponente.
   *
   * @return array
   */
  public function getAttributes()
  {
    return $this->_attributes;
  }

  /**
   * Fügt der Komponente eine CSS Klasse hinzu.
   *
   * @param string $class CSS Klasse
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addClass($class)
  {
    if(!in_array($class, $this->_classes))
      $this->_classes[] = $class;

    return $this;
  }

  /**
   * Entfernt eine CSS Klasse aus dieser Komponente.
   *
   * @param string $class CSS Klasse
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function removeClass($class)
  {
    $index = array_search($class, $this->_classes);
    if($index !== false)
      unset($this->_classes[$index]);

    return $this;
  }

  /**
   * Liefert die CSS Klassen dieser Komponente.
   *
   * @return array
   */
  public function getClasses()
  {
    return $this->_classes;
  }

  /**
   * Setzt eine CSS Eigenschaft dieser Komponente.
   *
   * @param string $style CSS Eigenschaft
   * @param string $value Wert der CSS Eigenschaft (null: die Eigenschaft wird entfernt)
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setStyle($style, $value)
  {
    if($value !== null)
      $this->_styles[$style] = $value;
    elseif(isset($this->_styles[$style]))
      unset($this->_styles[$style]);

    return $this;
  }

  /**
   * Liefert eine CSS Eigenschaft dieser Komponente.
   *
   * @param string $style CSS Eigenschaft
   * @return string
   */
  public function getStyle($style)
  {
    if (!isset($this->_styles[$style]))
      return null;
    else
      return $this->_styles[$style];
  }

  /**
   * Liefert die CSS Eigenschaften dieser Komponente.
   * @return array
   */
  public function getStyles()
  {
    return $this->_styles;
  }

  /**
   * Legt fest, ob die Komponente die restliche freie Breite einnehmen soll.
   *
   * @param boolean $fill true: die Komponente nimmt die restliche freie Breite ein
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setFillWidth($fill)
  {
    $this->_fillWidth = $fill;

    return $this;
  }

  /**
   * Liefert die Einstellung, ob die Komponente die restliche freie Breite
   * einnehmen soll oder nicht.
   *
   * @return boolean
   */
  public function getFillWidth()
  {
    return $this->_fillWidth;
  }

  /**
   * Legt fest, ob die Komponente die restliche freie Höhe einnehmen soll.
   *
   * @param boolean $fill true: die Komponente nimmt die restliche freie Höhe ein
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setFillHeight($fill)
  {
    $this->_fillHeight = $fill;

    return $this;
  }

  /**
   * Liefert die Einstellung, ob die Komponente die restliche freie Höhe
   * einnehmen soll oder nicht.
   *
   * @return boolean
   */
  public function getFillHeight()
  {
    return $this->_fillHeight;
  }

  /**
   * Setzt das Icon-Set aus dem Icons für diese Komponente verwendet werden.
   *
   * Komponenten, für die nicht explizit ein Icon-Set gesetzt wurde, verwenden
   * das Icon-Set des Containers, in dem sie enthalten sind (rekursiv bis
   * hinauf zur Webseiten-Komponente).
   *
   * @param GhGuiIconSet $iconSet Icon-Set
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIconSet(GhGuiIconSet $iconSet)
  {
    $this->_iconSet = $iconSet;

    return $this;
  }

  /**
   * Liefert das Icon-Set für diese Komponente.
   *
   * Falls diese Komponente kein explizites Icon-Set gesetzt hat, kann optional
   * der Container, in dem diese Komponente enthalten ist, nach einem Icon-Set
   * abgefragt werden (rekursiv bis hinauf zur Webseiten-Komponente).
   *
   * @param boolean $searchParents true: Icon-Set des Containers verwenden,
   *   false: nur Icon-Set dieser Komponente
   * @return GhGuiIconSet
   */
  public function getIconSet($searchParents = false)
  {
    if(!$searchParents || $this->_iconSet)
      return $this->_iconSet;

    $parent = $this->getParentContainer();
    if($parent)
      return $parent->getIconSet($searchParents);
    else
      return null;
  }

  /**
   * Setzt das Icon für diese Komponente.
   *
   * @param string $icon Name des Icons (aus dem Icon-Set)
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIcon($icon)
  {
    $this->_icon = $icon;

    return $this;
  }

  /**
   * Liefert das Icon für diese Komponente.
   *
   * @return string
   */
  public function getIcon()
  {
    return $this->_icon;
  }

  /**
   * Bindet einen Tooltip an diese Komponente.
   *
   * Der Tooltip wird angezeigt, wenn der Mauszeiger über die Komponente
   * bewegt wird. Falls ein GhGuiTooltipDialog als Tooltip verwendet wird, so
   * kann alternativ ein Event angegeben werden, durch das der Tooltip
   * angezeigt wird (z.B. 'onClick').
   *
   * @param GhGuiTooltip $tooltip Tooltip
   * @param string       $event   Event, durch das ein Tooltip-Dialog angezeigt wird (nur für GhGuiTooltipDialog relevant)
   * @return GhGuiTooltip liefert das Tooltip zurück, das an diese Komponente gebunden wurde
   */
  public function setTooltip(GhGuiTooltip $tooltip, $event = null)
  {
    $this->_tooltip = $tooltip;
    $this->_tooltipEvent = $event;

    return $tooltip;
  }

  /**
   * Liefert den Tooltip der an diese Komponente gebunden ist.
   *
   * @return GhGuiTooltip
   */
  public function getTooltip()
  {
    return $this->_tooltip;
  }

  /**
   * Liefert das Event, bei dem der Tooltip-Dialog angezeigt wird.
   *
   * @return string
   */
  public function getTooltipEvent()
  {
    return $this->_tooltipEvent;
  }

  /**
   * Setzt den Javascript-Klassennamen der Komponente.
   *
   * @param string $class Klassenname des Widgets
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  protected function _setWidgetClass($class)
  {
    $this->_widgetClass = $class;

    return $this;
  }

  /**
   * Liefert die von dieser Komponente benötigten Widget Klassen.
   *
   * Bei der Ausgabe der Seite werden automatisch entsprechende
   * 'dojo.require()' Aufrufe erzeugt.
   *
   * @return array
   */
  protected function _getRequiredWidgetClasses()
  {
    $widgetClasses = $this->_requiredWidgetClasses;

    if($this->_widgetClass && !in_array($this->_widgetClass, $this->_requiredWidgetClasses))
      $widgetClasses[] = $this->_widgetClass;

    return $widgetClasses;
  }

  /**
   * Setzt eine Eigenschaft für das aus dieser Komponente erzeugte Widget
   * Objekt.
   *
   * Die Eigenschaften 'class' und 'style' werden ausschließlich über die
   * Methoden addClass() bzw. setStyle() gesetzt.
   *
   * @param string $property Eigenschaft
   * @param string $value    Wert
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setWidgetProperty($property, $value)
  {
    $this->_widgetProperties[$property] = $value;

    return $this;
  }

  /**
   * Liefert den Wert einer Eigenschaft für das Widget Objekt dieser
   * Komponente.
   *
   * Die Eigenschaften 'class' und 'style' werden ausschließlich über die
   * Methoden getClasses() bzw. getStyle() abgerufen.
   *
   * @param string $property Eigenschaft
   * @return mixed
   */
  public function getWidgetProperty($property)
  {
    $properties = $this->getWidgetProperties();

    if(!isset($properties[$property]))
      return null;

    return $properties[$property];
  }

  /**
   * Liefert die Eigenschaften für das Widget Objekt dieser Komponente.
   *
   * @return array
   */
  public function getWidgetProperties()
  {
    $properties = $this->_widgetProperties;

    $icon = $this->getIcon();
    if($icon !== null)
    {
      $iconSet = $this->getIconSet(true);
      if($iconSet)
        $properties['iconClass'] = $iconSet->getIconSetClass() . ' ' . $icon;
    }

    return $properties;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    // Diese Liste enthält allgemeine Eigenschaften, die für mehrere Widgets
    // eine Rolle spielen. Spezielle Eigenschaften werden von abgeleiteten
    // Klassen ergänzt.
    switch($property)
    {
      case 'class':
      case 'design':
      case 'href':
      case 'iconClass':
      case 'id':
      case 'label':
      case 'region':
      case 'style':
      case 'title':
      case 'tooltip':
      case 'url':
        return 'string';

      case 'checked':
      case 'closable':
      case 'selected':
      case 'splitter':
        return 'boolean';

      case 'query':
        return 'object';
    }

    return null;
  }

  /**
   * Liefert die Objekteigenschaften für das Widget.
   *
   * @return array
   */
  protected function _getWidgetCreationProperties()
  {
    $properties = $this->getWidgetProperties();

    // CSS Klassen:
    $classes = $this->getClasses();
    if(!empty($classes))
      $properties['class'] = implode(' ', $classes);

    // CSS Eigenschaften:
    $styles = '';
    foreach($this->getStyles() as $style => $value)
      $styles .= $style . ':' . $value . ';';
    if(!empty($styles))
      $properties['style'] = $styles;

    return $properties;
  }

  /**
   * Liefert den Javascript Code zum Erzeugen eines Widget Objekts.
   *
   * Das Objekt erhält die Eigenschaften, die über setWidgetProperty() gesetzt
   * wurden, sowie die CSS Styles, die über setStyle() gesetzt wurden.
   * Optional können zusätzliche Eigenschaften übergeben werden. Diese
   * haben im Zweifelsfall Vorrang. Die zusätzlich übergebenen Eigenschaften
   * sind jeweils Arrays mit den Schlüsseln 'type' für den Wertetyp und 'value'
   * für den Wert der Eigenschaft.
   *
   * @param string $widgetClass  Klassenname für das Widget Objekt
   * @param array $properties Objekteigenschaften
   * @return string
   */
  protected function _getWidgetCreationJavascript($widgetClass = null, $properties = null)
  {
    if($widgetClass === null)
      $widgetClass = $this->_widgetClass;

    if($properties === null)
      $properties = $this->_getWidgetCreationProperties();

    $javascript = 'var ' . $this->getJavascriptWidgetVariable() . '=new ' . $widgetClass . '({';

    $jsonProperties = array();
    foreach($properties as $property => $value)
      $jsonProperties[] = GhCoreStrings::jsonProperty($property, $value, $this->__getWidgetPropertyType($property));
    $javascript .= implode(',', $jsonProperties);

    $javascript .= '},"' . $this->getId() . '");' . GhGui::$eol;

    return $javascript;
  }

  /**
   * Liefert zusätzliche, von dieser Komponente benötigte Javascript URLs.
   *
   * @return array
   */
  protected function _getRequiredJavascriptUrls()
  {
    return null;
  }

  /**
   * Fügt einen Event-Handler für ein Event zu dieser Komponente hinzu.
   *
   * @param string     $event   Name des Events
   * @param GhGuiEvent $handler Event-Handler
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addEventHandler($event, GhGuiEvent $handler)
  {
    if(!isset($this->_eventHandlers[$event]))
      $this->_eventHandlers[$event] = array($handler);
    elseif(!in_array($handler, $this->_eventHandlers[$event], true))
      $this->_eventHandlers[$event][] = $handler;

    return $this;
  }

  /**
   * Liefert alle Event-Handler dieser Komponente.
   *
   * Das Ergebnis ist ein Array, dessen Schlüssel Event-Namen und dessen Werte
   * Arrays mit Event-Handlern (Javascript-Funktionen) sind.
   *
   * @param string $event Name des Events (falls null werden alle Event-Handler zurückgeliefert)
   * @return array
   */
  public function getEventHandlers($event = null)
  {
    if($event === null)
      return $this->_eventHandlers;
    elseif(!isset($this->_eventHandlers[$event]))
      return array();
    else
      return $this->_eventHandlers[$event];
  }

  /**
   * Liefert die Javascript Variable, in der diese Komponente gespeichert ist.
   *
   * @return string
   */
  public function getJavascriptWidgetVariable()
  {
    return 'widget_' . $this->getId();
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if($this->_initHtmlDone)
      return false;

    $this->_initHtmlDone = true;

    if($this->_fillWidth || $this->_fillHeight)
    {
      $fillParams = ($this->_fillWidth ? 'true' : 'false') . ',' . ($this->_fillHeight ? 'true' : 'false');
      $this->addEventHandler('resize', new GhGuiEventJavascript('function(d){greyhound.dojo.fillSize(' . $this->getJavascriptWidgetVariable() . '.domNode,' . $fillParams . ');}'));
    }

    return true;
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    if(!$this->_htmlTag)
      return '';

    return GhGui::createHtml($this->_htmlTag, $this->_getAllAttributes());
  }

  /**
   * Liefert den Javascript Code für diese Komponente.
   *
   * Sofern diese Komponente einen Widget-Typ hat, wird ein entsprechendes Objekt
   * erzeugt.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    $javascript = '';

    if($this->_widgetClass)
      $javascript .= $this->_getWidgetCreationJavascript($this->_widgetClass);

    return $javascript;
  }

  /**
   * Liefert den Javascript Code für die Event-Handler dieser Komponente.
   *
   * @return string
   */
  protected function _getEventHandlersJavascript()
  {
    $javascript = '';

    foreach($this->getEventHandlers() as $event => $handlers)
    {
      foreach($handlers as $handler)
        $javascript .= $handler->getJavascript($this, $event);
    }

    return $javascript;
  }

  /**
   * Liefert Javascript Code für diese Komponente, der noch vor dem Erzeugen
   * der Komponenten ausgeführt wird.
   *
   * Dieser Code wird in dojo.ready() ausgeführt, bevor der Javascript-Code
   * den die Komponenten mit _getJavascript() liefern, ausgeführt wird.
   *
   * @return string
   */
  protected function _getJavascriptBeforeComponentCreation()
  {
    return '';
  }

  /**
   * Liefert Javascript Code für diese Komponente, der nach dem Erzeugen der
   * Komponenten ausgeführt wird.
   *
   * Dieser Code wird in dojo.ready() ausgeführt, nachdem der Javascript-Code
   * den die Komponenten mit _getJavascript() liefern, ausgeführt wird.
   *
   * @return string
   */
  protected function _getJavascriptAfterComponentCreation()
  {
    $javascript = '';

    if($this->_fillWidth || $this->_fillHeight)
    {
      $fillParams = ($this->_fillWidth ? 'true' : 'false') . ',' . ($this->_fillHeight ? 'true' : 'false');
      $javascript .= 'greyhound.dojo.fillSize(' . $this->getJavascriptWidgetVariable() . '.domNode,' . $fillParams . ');';
    }

    return $javascript;
  }

  /**
   * Liefert alle Attribute für den Haupt-DOM-Knoten dieser Komponente.
   *
   * Die Rückgabe enthält alle Attribute, die über die setAttribute() oder
   * setAttributes() Methode gesetzt wurden, sowie das 'style' Attribut, das
   * sich aus den Styles ergibt, die über setStyle() gesetzt wurden, sowie
   * einem 'id' Attribut mit der ID dieser Komponente.
   *
   * @return array
   */
  protected function _getAllAttributes()
  {
    $attributes = $this->getAttributes();

    // ID des HTML Knotens:
    $attributes['id'] = $this->_id;

    // CSS Klassen:
    $classes = $this->getClasses();
    if(!empty($classes))
      $attributes['class'] = implode(' ', $classes);

    // CSS Eigenschaften:
    $styles = '';
    foreach($this->getStyles() as $style => $value)
      $styles .= $style . ':' . GhCoreStrings::attributeValue($value) . ';';
    if(!empty($styles))
      $attributes['style'] = $styles;

    return $attributes;
  }

  /**
   * Liefert den Prozentwert einer Prozent-Größenangabe.
   *
   * Wenn die Größenangabe keine gültige Prozentangabe (mit Prozentzeichen)
   * ist wird null zurückgeliefert, ansonsten der numerische Prozentwert.
   *
   * @param string $size Größenangabe
   * @return int|float
   */
  protected function _getPercentValue($size)
  {
    if(!is_string($size))
      return null;
    $index = strpos($size, '%');
    if($index === null)
      return null;
    $size = trim(substr($size, 0, $index));
    if(!is_numeric($size))
      return null;
    return $size;
  }
}