<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Basisklasse für Container-Komponenten.
 *
 * @package GhGui
 * @subpackage core
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
abstract class GhGuiContainer extends GhGuiComponent
{
  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * Abgeleitete Klassen müssen diese Methode implementieren und eine Liste
   * aller Komponenten, die in der Klasse enthalten sind, zurück liefern.
   * Diese Liste wird verwendet, um notwendige Operationen (z.B.
   * Initialisierung, Sammeln von Javascript-Code) auf allen Komponenten
   * durchzuführen.
   *
   * @return array
   */
  abstract protected function _getAllComponents();

  /**
   * Liefert true falls dieser Container Komponenten enthält und false falls
   * er leer ist (keine Komponenten enthält).
   *
   * @return boolean
   */
  public function hasComponents()
  {
    $components = $this->_getAllComponents();
    return !empty($components);
  }

  /**
   * Liefert alle Icon-Sets, die von diesem Container oder den darin
   * enthaltenen Komponenten verwendet werden.
   *
   * @return array
   */
  protected function _getAllIconSets()
  {
    $iconSets = array();

    $iconSet = $this->getIconSet(false);
    if($iconSet)
      $iconSets[$iconSet->getIconSetClass()] = $iconSet;

    foreach($this->_getAllComponents() as $component)
    {
      if($component instanceof GhGuiContainer)
        $iconSets = array_merge($iconSets, $component->_getAllIconSets());
      else
      {
        /* @var GhGuiComponent $component */
        $iconSet = $component->getIconSet(false);
        if($iconSet)
          $iconSets[$iconSet->getIconSetClass()] = $iconSet;
      }
    }

    return $iconSets;
  }

  /**
   * Liefert alle Tooltips, die von diesem Container oder den darin
   * enthaltenen Komponenten verwendet werden.
   *
   * Die Rückgabe ist ein Array, dessen Schlüssel die IDs der Tooltips sind
   * und dessen Werte Arrays mit dem Schlüssel 'tooltip' und dem
   * Tooltip-Objekt, sowie 'ids' und einem Array mit IDs der Komponenten, an
   * die dieser Tooltip gebunden ist.
   *
   * @return array
   */
  protected function _getTooltipsAndIds()
  {
    $tooltips = array();

    $tooltip = $this->getTooltip();

    if($tooltip)
      $tooltips[$tooltip->getId()] = array('tooltip' => $tooltip, 'components' => array($this->getId() => $this));

    foreach($this->_getAllComponents() as $component)
    {
      if($component instanceof GhGuiContainer)
      {
        /* @var GhGuiContainer $component */
        foreach($component->_getTooltipsAndIds() as $tooltipId => $tooltipData)
        {
          if(isset($tooltips[$tooltipId]))
            $tooltips[$tooltipId]['components'] = array_merge($tooltips[$tooltipId]['components'], $tooltipData['components']);
          else
            $tooltips[$tooltipId] = $tooltipData;
        }
      }
      else
      {
        /* @var GhGuiComponent $component */
        $tooltip = $component->getTooltip();
        if($tooltip)
        {
          if(isset($tooltips[$tooltip->getId()]))
            $tooltips[$tooltip->getId()]['components'][$component->getId()] = $component;
          else
            $tooltips[$tooltip->getId()] = array('tooltip' => $tooltip, 'components' => array($component->getId() => $component));
        }
      }
    }

    return $tooltips;
  }

  /**
   * Initialisiert die Komponente und alle in ihr enthaltenen Komponenten für
   * das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $components = $this->_getAllComponents();
    foreach($components as $component)
    {
      /* @var GhGuiComponent $component */
      // Sicherstellen, dass alle Parent-Container gesetzt sind:
      $component->_setParentContainer($this);
      // Initialisierungs-Code hinzufügen:
      $component->_initHtml();
    }

    // Style-Klasse für Elemente, die alleine (sichtbar) in einem Container enthalten sind:
    $visible = null;

    foreach($components as $component)
    {
      /* @var GhGuiComponent $component */
      if($component->getStyle('display') != 'none')
      {
        if($visible === null)
          $visible = $component;
        else
        {
          $visible = null;
          break;
        }
      }
    }

    if($visible !== null)
      $visible->addClass('GhGuiIsSingleChild');

    return true;
  }

  /**
   * Liefert den HTML Code für diese Komponente und die darin enthaltenen
   * Komponenten.
   *
   * @return string
   */
  protected function _getHtml()
  {
    $html = '';

    foreach ($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $html .= $component->_getHtml();

    if($this->_htmlTag)
      $html = GhGui::createHtml($this->_htmlTag, $this->_getAllAttributes(), $html);

    return $html;
  }

  /**
   * Liefert zusätzliche, von dieser Komponente und ihren enthaltenen
   * Komponenten benötigte Javascript URLs.
   *
   * @return array
   */
  protected function _getRequiredJavascriptUrls()
  {
    $urls = parent::_getRequiredJavascriptUrls();

    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
    {
      $componentUrls = $component->_getRequiredJavascriptUrls();

      if(is_array($componentUrls))
      {
        if(is_array($urls))
          $urls = array_merge($urls, $componentUrls);
        else
          $urls = $componentUrls;
      }
    }

    if(is_array($urls))
      $urls = array_unique($urls);

    return $urls;
  }

  /**
   * Liefert die von dieser Komponente und ihren enthaltenen Komponenten
   * benötigten Widget Klassen.
   *
   * @return array
   */
  protected function _getRequiredWidgetClasses()
  {
    $classes = parent::_getRequiredWidgetClasses();

    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $classes = array_unique(array_merge($classes, $component->_getRequiredWidgetClasses()));

    return $classes;
  }

  /**
   * Liefert den Javascript Code für diese Komponente und die darin enthaltenen
   * Komponenten.
   *
   * Sofern diese Komponente einen Widget-Typ hat, wird ein entsprechendes Objekt
   * erzeugt.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    $javascript = parent::_getJavascript();

    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getJavascript();

    return $javascript;
  }

  /**
   * Liefert den Javascript Code für die Event-Handler dieser Komponente und
   * der darin enthaltenen Komponenten.
   *
   * @return string
   */
  protected function _getEventHandlersJavascript()
  {
    $javascript = parent::_getEventHandlersJavascript();

    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getEventHandlersJavascript();

    return $javascript;
  }

  /**
   * Liefert Javascript Code für diese Komponente und die darin enthaltenen
   * Komponenten, der noch vor dem Erzeugen der Komponenten ausgeführt wird.
   *
   * Dieser Code wird in dojo.ready() ausgeführt, bevor der Javascript-Code
   * den die Komponenten mit _getJavascript() liefern, ausgeführt wird.
   *
   * @return string
   */
  protected function _getJavascriptBeforeComponentCreation()
  {
    $javascript = '';
    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getJavascriptBeforeComponentCreation();

    return $javascript;
  }

  /**
   * Liefert Javascript Code für diese Komponente und die darin enthaltenen
   * Komponenten, der nach dem Erzeugen der Komponenten ausgeführt wird.
   *
   * Dieser Code wird in dojo.ready() ausgeführt, nachdem der Javascript-Code
   * den die Komponenten mit _getJavascript() liefern, ausgeführt wird.
   *
   * @return string
   */
  protected function _getJavascriptAfterComponentCreation()
  {
    $javascript = '';
    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getJavascriptAfterComponentCreation();

    return $javascript;
  }
}
