<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Basisklasse für Webseiten.
 *
 * Diese Klasse dient als Basis für Klassen, die die Darstellung der kompletten
 * Webseite vornehmen.
 *
 * Klassen, die von dieser Klasse abgeleitet werden müssen die Methode
 * _getAllComponents() überladen um alle auf der Seite enthaltenen Komponenten
 * zurück zu liefern, und die Methode _getHtml() um dort den gesamten Code für
 * die Webseite zurück liefern, vom DOCTYPE und öffnenden HTML-Tag bis zum
 * schließenden HTML-Tag.
 *
 * @package GhGui
 * @subpackage core
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
abstract class GhGuiPage extends GhGuiContainer
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = null;

  /**
   * Zusätzliche Zeilen (für den 'head' Knoten der Webseite).
   * @var array
   */
  protected $_head = array();

  /**
   * Titel der Webseite.
   * Dieser Wert wird für das 'title' Tag im Webseitenkopf verwendet.
   * @var string
   */
  protected $_title;

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('greyhound');

  /**
   * Eingebundene Javascript Dateien.
   * URLs relativ zur Basis-URL der Webseite können in der Form
   * 'ghgui://lib/greyhound/...' angegeben werden.
   * @var array
   */
  protected $_javascripts = array(
    'ghgui://lib/dojo/dojo.js',
    'ghgui://lib/dijit/dijit.js',
    'ghgui://lib/dijit/dijit-all.js',
    'ghgui://lib/greyhound/greyhound.js'
  );

  /**
   * CSS Stylesheets der Webseite.
   * URLs relativ zur Basis-URL der Webseite können in der Form
   * 'ghgui://lib/greyhound/...' angegeben werden.
   * @var array
   */
  protected $_stylesheets = array(
    'ghgui://lib/greyhound/dojo/theme/greyhound.css',
    'ghgui://lib/greyhound/dojo/theme/grid/Grid.css',
    'ghgui://lib/greyhound/dojo/theme/grid/greyhoundGrid.css',
  );

  /**
   * Ausgabemedien für Stylesheets, deren Ausgabemedium von "screen" abweicht.
   * @var array
   */
  protected $_stylesheetsMedia = array();

  /**
   * Zusätzliche CSS Definitionen.
   * @var array
   */
  protected $_css = array();

  /**
   * Konfigurationseinstellungen für die Widget-Bibliothek.
   * @var array
   */
  protected $_widgetConfig = array(
    'defaultDuration' => 1
  );

  /**
   * Standard Icon-Set.
   * @var GhGuiIconSet
   */
  protected $_defaultIconSet;

  /**
   * Komponenten, die in die Seite eingebunden werden, aber keiner bestimmten
   * Position zugeordnet werden (z.B. Tooltips).
   * @var array
   */
  protected $_globalComponents = array();

  /**
   * Legt fest, ob die Seite während des Ladens bzw. Aufbaus einen
   * Splash-Screen anzeigt.
   * Dier Splash-Screen überlagert die restlichen Seiteninhalte und verhindert
   * so, dass der Prozess des Webseitenaufbaus sichtbar ist.
   * @var boolean
   */
  protected $_useSplashScreen = true;

  /**
   * CSS Stilangaben für den Splash-Screen.
   * @var array
   */
  protected $_splashScreenStyles = array('background' => '#6e6e6e', 'position' => 'absolute', 'top' => '0px', 'left' => '0px', 'width' => '100%', 'height' => '100%', 'margin' => '0px', 'padding' => '0px', 'border' => '0px', 'z-index' => '9999');

  /**
   * Inhalt (HTML) des Splash-Screens.
   * @var string
   */
  protected $_splashScreenContent = '';

  /**
   * ID des Splash-Screen DOM-Knotens.
   * @var string
   */
  protected $_splashScreenId;

  /**
   * Webseiten-Icon URL (Favicon).
   * @var string
   */
  protected $_favIconUrl = 'ghgui://lib/greyhound/favicon.ico';

  /**
   * Zusätzlicher Javascript Code, der nach dem Erzeugen der Komponenten ausgeführt wird.
   * @var string
   */
  protected $_additionalJavascript = '';

  /**
   * Die Komponente, die beim Anzeigen der Seite den Eingabefokus erhält.
   * @var GhGuiComponent
   */
  protected $_focusComponent;

  /**
   * Tastatur-Kommandos.
   * Die Einträge dieses Arrays sind Arrays, deren erstes Element der
   * Tastencode, das zweite Element zusätzliche Sondertasten und das dritte
   * Element der Event-Handler.
   * @var array
   */
  protected $_keyboardShortcuts = array();

  /**
   * Konstruktor.
   *
   * Optional kann der Titel der Webseite (für den 'title' Knoten) und die
   * Basis-URL der Webseite angegeben werden.
   *
   * @param string $title   Titel der Webseite
   * @param string $baseUrl Basis-URL der Webseite
   */
  public function __construct($title = null, $baseUrl = null)
  {
    parent::__construct();

    $this->_splashScreenId = GhGui::createId();

    // Microsoft Internet Explorer Kompatiblitätseinstellung:
    // Dies ist derzeit notwendig um Layout-Probleme im Internet Explorer zu vermeiden.
    // Ggf. wird diese Option in der Zukunft geändert oder entfernt, wenn das Layout vollständig mit Internet Explorer 9 (oder neuer) funktioniert.
    $this->_head[] = GhGui::createHtml('meta', array('http-equiv' => 'X-UA-Compatible', 'content' => 'IE=EmulateIE8'));

    if($title !== null)
      $this->setTitle($title);

    if($baseUrl !== null)
      GhGui::setBaseUrl($baseUrl);

    $this->setIconSet($this->getDefaultIconSet());
  }

  /**
   * Liefert den HTML Code für diese Komponente und die darin enthaltenen
   * Komponenten.
   *
   * Diese Methode muss überladen werden und den gesamten Code für die Webseite
   * zurück liefern, vom DOCTYPE und öffnenden HTML-Tag bis zum schließenden
   * HTML-Tag.
   *
   * Im Seitenkopf müssen insbesondere die Kopfzeilen (getHead()), die
   * Javascript- und Stylesheet-Dateien ($this->_javascriptUrls,
   * $this->_stylesheetUrls), den zusätzlichen Javascript Code
   * ($this->_additionalJavascript), die CSS Angaben (_getCss()), sowie der
   * Javascript Code zum Erstellen aller Komponenten der Webseite (inkl. der
   * Komponenten in $this->_globalComponents) ausgegeben werden.
   *
   * Im Seitenrumpf müssen alle Komponenten der Webseite (inkl. der Komponenten
   * in $this->_globalComponents()) ausgegeben werden.
   *
   * @return string
   */
  protected function _getHtml()
  {
  }
  
  /**
   * Setzt den Titel der Webseite.
   * Dieser Wert wird für das 'title' Tag im Webseitenkopf verwendet.
   *
   * @param string $title Titel der Webseite
   * @return GhGuiExtensionPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTitle($title)
  {
    $this->_title = $title;

    return $this;
  }

  /**
   * Liefert den Titel der Webseite.
   * @return string
   */
  public function getTitle()
  {
    return $this->_title;
  }

  /**
   * Setzt die URL des Webseiten Icons (Fav-Icon).
   *
   * @param string $url Fav-Icon URL
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setFavIconUrl($url)
  {
    $this->_favIconUrl = $url;

    return $this;
  }

  /**
   * Liefert die URL des Webseiten Icons (Fav-Icon).
   *
   * @return string
   */
  public function getFavIconUrl()
  {
    return $this->_favIconUrl;
  }

  /**
   * Legt die Komponente fest, die beim Anzeigen der Webseite den Fokus erhält.
   *
   * @param GhGuiComponent $component die Komponente, die den Fokus erhält
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setFocus(GhGuiComponent $component)
  {
    $this->_focusComponent = $component;

    return $this;
  }

  /**
   * Liefert die Komponente, die beim Anzeigen der Webseite den Fokus erhält.
   *
   * @return GhGuiComponent
   */
  public function getFocus()
  {
    return $this->_focusComponent;
  }

  /**
   * Fügt ein Tastatur-Kommando hinzu.
   *
   * @param string|int $key       Zeichen oder Tastencode
   * @param int        $modifiers Zusatztasten
   * @param GhGuiEvent $event     Event-Handler
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addKeyboardShortcut($key, $modifiers, GhGuiEvent $event)
  {
    $this->_keyboardShortcuts[] = array($key, $modifiers, $event);

    return $this;
  }

  /**
   * Fügt eine Zeile zum 'head'-Knoten der Webseite hinzu.
   *
   * @param string $line Header-Zeile
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addToHead($line)
  {
    $this->_head[] = $line;

    return $this;
  }

  /**
   * Entfernt eine Zeile vom 'head'-Knoten der Webseite.
   *
   * @param string $line Header-Zeile
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function removeFromHead($line)
  {
    foreach($this->_head as $key => $tmpLine)
    {
      if($line == $tmpLine)
        unset($this->_head[$key]);
    }

    return $this;
  }

  /**
   * Liefert die Zeilen für den 'head'-Knoten der Webseite.
   *
   * @return array
   */
  public function getHead()
  {
    $head = $this->_head;

    $head[] = $this->_getHeadCharset() . GhGui::$eol;
    $head[] = $this->_getHeadTitle() . GhGui::$eol;
    $head[] = $this->_getHeadFavIcon() . GhGui::$eol;

    foreach($this->_getHeadStylesheets() as $headLine)
      $head[] = $headLine . GhGui::$eol;

    $cssHead = $this->_getHeadCss();

    if(!empty($cssHead))
      $head[] = $cssHead . GhGui::$eol;

    foreach($this->_getHeadJavascripts() as $headLine)
      $head[] = $headLine . GhGui::$eol;

    return $head;
  }

  /**
   * Fügt ein CSS Stylesheet zur Webseite hinzu.
   *
   * @param string $url   Stylesheet URL
   * @param string $media Ausgabemedien auf die das Stylesheet angewendet wird
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addStylesheetUrl($url, $media = 'screen')
  {
    $this->_stylesheets[] = $url;

    if($media != 'screen')
      $this->_stylesheetsMedia[$url] = $media;

    return $this;
  }

  /**
   * Entfernt ein CSS Stylesheet aus dieser Webseite.
   *
   * @param string $url Stylesheet URL
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function removeStylesheetUrl($url)
  {
    foreach($this->_stylesheets as $key => $stylesheet)
    {
      if($url == $stylesheet)
        unset($this->_stylesheets[$key]);
    }

    if(isset($this->_stylesheetsMedia[$url]))
      unset($this->_stylesheetsMedia[$url]);

    return $this;
  }

  /**
   * Liefert die CSS Stylesheets dieser Webseite.
   * 
   * @return array
   */
  public function getStylesheetUrls()
  {
    return $this->_stylesheets;
  }

  /**
   * Liefert die Ausgabemedien auf die ein Stylesheet angewendet wird.
   *
   * @param string $url Stylesheet-URL
   * @return string
   */
  public function getStylesheetMedia($url)
  {
    if(isset($this->_stylesheetsMedia[$url]))
      return $this->_stylesheetsMedia[$url];
    else
      return 'screen';
  }

  /**
   * Fügt eine Javascript Datei zur Webseite hinzu.
   *
   * @param string $url Javascript URL
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addJavascriptUrl($url)
  {
    $this->_javascripts[] = $url;

    return $this;
  }

  /**
   * Entfernt eine Javascript Datei aus dieser Webseite.
   *
   * @param string $url Javascript URL
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function removeJavascriptUrl($url)
  {
    foreach($this->_javascripts as $key => $javascript)
    {
      if($url == $javascript)
        unset($this->_javascripts[$key]);
    }

    return $this;
  }

  /**
   * Liefert die Javascript Dateien dieser Webseite.
   *
   * @return array
   */
  public function getJavascriptUrls()
  {
    return $this->_javascripts;
  }

  /**
   * Fügt zusätzlichen Javascript Code hinzu, der nach dem Erzeugen der
   * Komponenten ausgeführt wird.
   *
   * @param string $javascript Javascript Code
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addJavascriptCode($javascript)
  {
    $this->_additionalJavascript .= $javascript;

    return $this;
  }

  /**
   * Fügt CSS Definitionen zur Webseite hinzu.
   *
   * Die mit dieser Methode hinzugefügten Definitionen werden in einem 'style'
   * HTML Knoten im Webseitenkopf eingefügt.
   *
   * @param string $css CSS Definitionen
   * @return GhGuiPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addCss($css)
  {
    $this->_css[] = $css;

    return $this;
  }

  /**
   * Liefert CSS Style-Angaben für die Webseite.
   *
   * Diese Angaben sollten im Seitenkopf in einem style-Knoten ausgegeben
   * werden und enthalten beispielsweise die Style-Eigenschaften für die
   * Icon-Sets der Webseite.
   *
   * @return string
   */
  protected function _getCss()
  {
    $css = '';

    foreach($this->_getAllIconSets() as $iconSet)/* @var GhGuiIconSet $iconSet */
    {
      $iconSetCss = $iconSet->getCss();

      if(!empty($iconSetCss))
        $css .= "\r\n" . $iconSetCss;
    }

    foreach($this->_css as $cssDefinitions)
      $css .= "\r\n" . $cssDefinitions;

    return $css;
  }

  /**
   * Liefert die Kopfzeile für die Zeichencodierung.
   *
   * @return string
   */
  protected function _getHeadCharset()
  {
    return GhGui::createHtml('meta', array('http-equiv' => 'Content-Type', 'content' => 'text/html; charset=' . GhGui::getCharset()));
  }

  /**
   * Liefert die Kopfzeile für den Seitentitel.
   *
   * @return string
   */
  protected function _getHeadTitle()
  {
    return GhGui::createHtml('title', null, $this->getTitle());
  }

  /**
   * Liefert die Kopfzeile für das Webseiten Icon (Fav-Icon).
   *
   * @return string
   */
  protected function _getHeadFavIcon()
  {
    return GhGui::createHtml('link', array('rel' => 'Shortcut Icon', 'type' => 'image/x-icon', 'href' => GhGui::createUrl($this->getFavIconUrl())));
  }

  /**
   * Liefert die Kopfzeilen für die CSS Stylesheet Dateien.
   *
   * @return array
   */
  protected function _getHeadStylesheets()
  {
    $head = array();

    foreach($this->getStylesheetUrls() as $stylesheet)
      $head[] = GhGui::createHtml('link', array('rel' => 'stylesheet', 'type' => 'text/css', 'media' => $this->getStylesheetMedia($stylesheet), 'href' => GhGui::createUrl($stylesheet)));

    return $head;
  }

  /**
   * Liefert die Kopfzeile für die CSS Angaben der Seite.
   *
   * @return string
   */
  protected function _getHeadCss()
  {
    $css = $this->_getCss();

    if(!empty($css))
      return GhGui::createHtml('style', array('type' => 'text/css'), $css);
    else
      return null;
  }

  /**
   * Liefert die von dieser Komponente und den in ihr enthaltenen Komponenten
   * benötigten Javascript URLs.
   *
   * @return array
   */
  protected function _getRequiredJavascriptUrls()
  {
    $urls = $this->getJavascriptUrls();

    $addUrls = parent::_getRequiredJavascriptUrls();

    if(is_array($addUrls))
      $urls = array_merge($urls, $addUrls);

    $urls = array_unique($urls);

    return $urls;
  }

  /**
   * Liefert die Kopfzeilen für Javascript Dateien und Javascript Code.
   *
   * @return array
   */
  protected function _getHeadJavascripts()
  {
    $head = array();

    // Javascript-Dateien einbinden:
    foreach($this->_getRequiredJavascriptUrls() as $javascriptUrl)
      $head[] = GhGui::createHtml('script', array('type' => 'text/javascript', 'src' => GhGui::createUrl($javascriptUrl))) . GhGui::$eol;

    $javascript = '';

    // Benötigte Widget Klassen:
    $widgetClasses = $this->_getRequiredWidgetClasses();

    if(!empty($widgetClasses))
      $javascript .= 'dojo.require("' . implode('");' . GhGui::$eol . 'dojo.require("', $widgetClasses) . '");' . GhGui::$eol;

    // Javascript vor dem Erzeugen der Komponenten:
    $javascript .= 'greyhound.baseUrl="' . GhGui::getBaseUrl() . '";' . GhGui::$eol;

    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getJavascriptBeforeComponentCreation();

    // Seiten-Hauptkomponenten erzeugen:
    $javascript .= $this->_getHeadCreatePageComponentsJavascript();

    // Javascript zum Erzeugen/Initialisieren der Komponenten:
    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getJavascript();

    // Seiten-Hauptkomponenten initialisieren bzw. anzeigen:
    $javascript .= $this->_getHeadInitPageComponentsJavascript();

    // Javascript nach dem Erzeugen der Komponenten:
    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getJavascriptAfterComponentCreation();

    // Splash-Screen verbergen:
    $javascript .= $this->_getSplashScreenHideJavascript();

    // Javascript für Event-Handler:
    foreach($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $javascript .= $component->_getEventHandlersJavascript();

    // Fokus festlegen:
    $focus = $this->getFocus();
    if($focus)
      $javascript .= 'if(' . $focus->getJavascriptWidgetVariable() . '.focus)' . $focus->getJavascriptWidgetVariable() . '.focus();' . GhGui::$eol;

    // Zusätzlicher Javascript Code:
    $javascript .= $this->_additionalJavascript;

    // Javascript Code zum Kopf hinzufügen:
    if($javascript)
      $head[] = GhGui::createHtml('script', array('type' => 'text/javascript'), 'dojo.ready(function(){' . GhGui::$eol . GhGui::$eol . $javascript . GhGui::$eol . '});' . GhGui::$eol);

    return $head;
  }

  /**
   * Liefert den Javascript Code zum Erzeugen der Komponenten, in denen alle
   * Komponenten der Webseite enthalten sind.
   *
   * Falls die Webseite HTML Knoten enthält, in denen sich Komponenten
   * befinden, so müssen für diese Widgets erzeugt werden.
   *
   * @return string
   */
  protected function _getHeadCreatePageComponentsJavascript()
  {
    return '';
  }

  /**
   * Liefert den Javascript Code zum Initialisieren bzw. Anzeigen der
   * Komponenten, in denen alle Komponenten der Webseite enthalten sind.
   *
   * Alle Widgets müssen initialisiert werden. Da Widgets die in ihnen
   * enthaltenen Widgets selbstständig initialisieren, müssen alle Widgets der
   * obersten Ebene einmal explizit initialisiert werden um die gesamte
   * Webseite zu initialisieren. Dazu wird auf allen Widgets der obersten Ebene
   * die Javascript Methode ##startup()## aufgerufen. Andernfalls sind die
   * Widgets nicht sichtbar.
   *
   * @return string
   */
  protected function _getHeadInitPageComponentsJavascript()
  {
    return '';
  }

  /**
   * Setzt den Container, in dem diese Komponente enthalten ist.
   *
   * Diese Methode wird von einem Container aufgerufen wenn eine Komponente
   * in ihn eingefügt wird.
   *
   * Diese Methode wirft bei GhGuiPage Webseiten eine GhGuiException, da
   * Webseiten nicht als Komponente zu anderen Containern hinzugefügt werden
   * können.
   *
   * @throws GhGuiException wird immer geworfen, da GhGuiPage Komponenten nicht in andere Container eingefügt werden können.
   *
   * @param GhGuiContainer $container der Container, der diese Komponente enthält
   * @return GhGuiComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  protected function _setParentContainer(GhGuiContainer $container = null)
  {
    throw new GhGuiException('GhGuiPage components may not be added to any other components');
  }

  /**
   * Liefert den gesamten Webseiten-Inhalt (inkl. DOCTYPE, Javascript, HTML).
   *
   * @return string
   */
  public function getHtmlPage()
  {
    $this->_initHtml();

    $html = $this->_getHtml();

    return $html;
  }

  /**
   * Gibt die Webseite aus.
   */
  public function show()
  {
    echo $this->getHtmlPage();
  }

  /**
   * Initialisiert die Komponente und alle in ihr enthaltenen Komponenten für
   * das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    if(GhGui::getDebugMode())
    {
      // Debug-Modus der Widget-Bibliothek aktivieren:
      $this->_widgetConfig['isDebug'] = true;
      
      // Im Debug-Modus das unkomprimierte Dojo-Javascript einbinden:
      foreach($this->_javascripts as $key => $javascript)
      {
        if($javascript == 'ghgui://lib/dojo/dojo.js' || $javascript == 'ghgui://lib/dijit/dijit.js' || $javascript == 'ghgui://lib/dijit/dijit-all.js')
          $this->_javascripts[$key] = $javascript . '.uncompressed.js';
      }
    }

    // Konfiguration der Widget-Bibliothek in den 'head'-Knoten aufnehmen:
    if(!empty($this->_widgetConfig))
    {
      $config = array();
      foreach($this->_widgetConfig as $key => $value)
        $config[] = GhCoreStrings::jsonProperty($key, $value);
      $javascript = 'var djConfig={' . implode(',', $config) . '};';
      $this->addToHead(GhGui::createHtml('script', array('type' => 'text/javascript'), $javascript));
    }

    // Stylesheet Dateien der Icon-Sets hinzufügen:
    foreach($this->_getAllIconSets() as $iconSet)/* @var GhGuiIconSet $iconSet */
    {
      $stylesheet = $iconSet->getStylesheetUrl();
      if($stylesheet && !in_array($stylesheet, $this->_stylesheets))
      {
        $this->_stylesheets[] = $stylesheet;
        $this->_stylesheetsMedia[$stylesheet] = 'screen,print';
      }
    }

    // Tooltips zu den globalen Komponenten hinzufügen und die Komponenten-IDs setzen:
    foreach($this->_getTooltipsAndIds() as $tooltipData)
    {
      $tooltip = $tooltipData['tooltip'];

      if($tooltip instanceof GhGuiTooltipDialog)
      {
        switch($tooltip->getTooltipPosition())
        {
          case GhGuiTooltipDialog::POSITION_TOP:
            $position = ',orient:{TL:"BL"}';
            break;

          case GhGuiTooltipDialog::POSITION_RIGHT:
            $position = ',orient:{BR:"BL"}';
            break;

          case GhGuiTooltipDialog::POSITION_BOTTOM:
            $position = ',orient:{BL:"TL"}';
            break;

          case GhGuiTooltipDialog::POSITION_LEFT:
            $position = ',orient:{BL:"BR"}';
            break;

          default:
            $position = '';
            break;
        }

        foreach($tooltipData['components'] as $component)
        {
          /* @var GhGuiComponent $component */
          $event = $component->getTooltipEvent();

          if(!$event)
            $event = 'onMouseOver';

          $component->addEventHandler($event, new GhGuiEventJavascript('function(event){dijit.popup.open({parent:' . $component->getJavascriptWidgetVariable() . $position . ',popup:' . $tooltip->getJavascriptWidgetVariable() . ',around:' . $component->getJavascriptWidgetVariable() . '.domNode,onClose:function(){dijit.popup.close(' . $tooltip->getJavascriptWidgetVariable() . ');}});' . $tooltip->getJavascriptWidgetVariable() . '.focus();}'));
        }
      }
      else
      {
        /* @var GhGuiComponent $tooltip */
        $tooltip->setWidgetProperty('connectId', array_keys($tooltipData['components']));
      }

      $this->_globalComponents[] = $tooltip;
    }

    foreach($this->_globalComponents as $component)
      $component->_setParentContainer($this);

    // Alle Komponenten der Webseite initialisieren:
    foreach($this->_getAllComponents() as $component)
      $component->_initHtml();

    // Tastatur-Kommandos:
    if(!empty($this->_keyboardShortcuts))
    {
      $javascript = 'function(event){';
      $javascript .= 'var k="";var m=0;if((event.keyCode>64&&event.keyCode<91)||(event.keyCode>47&&event.keyCode<58))k=String.fromCharCode(event.keyCode);else k=event.keyCode;';
      $javascript .= 'if(event.altKey)m+=' . GhGuiEvent::MODIFIER_ALT . ';';
      $javascript .= 'if(event.altGraphKey)m+=' . GhGuiEvent::MODIFIER_ALT_GRAPH . ';';
      $javascript .= 'if(event.ctrlKey)m+=' . GhGuiEvent::MODIFIER_CTRL . ';';
      $javascript .= 'if(event.metaKey)m+=' . GhGuiEvent::MODIFIER_META . ';';
      $javascript .= 'if(event.shiftKey)m+=' . GhGuiEvent::MODIFIER_SHIFT . ';';

      foreach($this->_keyboardShortcuts as $shortcut)
      {
        $javascript .= 'if(k==';

        if(substr($shortcut[0], 0, 10) == 'dojo.keys.')
          $javascript .= $shortcut[0];
        else
          $javascript .= '"' . strtoupper($shortcut[0]) . '"';

        $javascript .= '&&m==' . (int)$shortcut[1] . ')';
        $shortcutEvent = $shortcut[2];/* @var GhGuiEvent $shortcutEvent */
        $javascript .= '{var fnc=' . $shortcutEvent->getJavascriptFunction($this, 'onkeypress') . ';fnc(event);dojo.stopEvent(event);}';
      }

      $javascript .= '}';
      $this->addJavascriptCode('dojo.connect(document,"onkeydown",' . $javascript . ');');
    }

    return true;
  }

  /**
   * Liefert den HTML Code zum Erzeugen des Splash-Screens.
   *
   * @return string
   */
  protected function _getSplashScreenHtml()
  {
    if(!$this->_useSplashScreen)
      return '';

    $style = '';

    foreach($this->_splashScreenStyles as $key => $value)
      $style .= $key . ':' . $value . ';';

    $style = GhCoreStrings::attributeValue($style);

    return GhGui::createHtml('div', array('id' => $this->_splashScreenId, 'style' => $style));
  }

  /**
   * Liefert den Javascript Code zum Verbergen des Splash-Screens.
   *
   * @return string
   */
  protected function _getSplashScreenHideJavascript()
  {
    if(!$this->_useSplashScreen)
      return '';

    return 'dojo.style("' . $this->_splashScreenId . '","display","none");' . GhGui::$eol;
  }

  /**
   * Liefert einen leeren String.
   *
   * Diese Methode liefert einen leeren String, da der Javascript-Code bereits
   * in der HTML-Ausgabe der Webseite enthalten sein muss.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    return '';
  }

  /**
   * Liefert einen leeren String.
   *
   * Diese Methode liefert einen leeren String, da der Javascript-Code bereits
   * in der HTML-Ausgabe der Webseite enthalten sein muss.
   *
   * @return string
   */
  protected function _getEventHandlersJavascript()
  {
    return '';
  }

  /**
   * Liefert das Standard Icon-Set.
   *
   * @return GhGuiIconSet
   */
  public function getDefaultIconSet()
  {
    if(!$this->_defaultIconSet)
      $this->_defaultIconSet = new GhGuiDefaultIconSet();

    return $this->_defaultIconSet;
  }
}
