<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Event-Handler, der den Wert eines versteckten Formularfelds setzt.
 *
 * @package GhGui
 * @subpackage event
 * @category PHP-GUI-Event-Handler
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiEventSetHiddenField extends GhGuiEvent
{
  /**
   * Name des versteckten Formularfelds.
   * @var string
   */
  protected $_name;

  /**
   * Wert des versteckten Formularfelds.
   * @var string
   */
  protected $_value;

  /**
   * Legt fest, ob der Wert für das versteckte Formularfeld Javascript-Code
   * ist.
   * @var boolean
   */
  protected $_valueIsJavascript;

  /**
   * Formular, zu dem das versteckte Formularfeld hinzugefügt wird.
   * @var GhGuiForm
   */
  protected $_form;

  /**
   * Konstruktor.
   *
   * Der Name und der Wert des versteckten Formularfelds müssen angegeben werden. Optional kann zudem die
   * Formularkomponente angegeben werden. Wird diese nicht gesetzt, so wird das versteckte Formularfeld dem Formular
   * hinzugefügt, in dem die Komponente liegt, auf der dieser Event-Handler registriert wird.
   *
   * @param string|GhGuiHiddenField $name              Name des Formularfelds
   * @param string                  $value             Wert des Formularfelds
   * @param boolean                 $valueIsJavascript true: der Wert ist Javascript-Code, false: der Wert wird als String übernommen
   * @param GhGuiForm               $form              Formularkomponente
   */
  public function __construct($name, $value, $valueIsJavascript = null, GhGuiForm $form = null)
  {
    parent::__construct();

    $this->_name = $name;
    $this->_value = $value;
    $this->_valueIsJavascript = $valueIsJavascript;
    $this->_form = $form;
  }

  /**
   * Liefert den Javascript Code für die Event-Hanlder Javascript-Funktion.
   *
   * @param GhGuiComponent $source Komponente, auf der das Event registriert werden soll
   * @param string         $event  Event-Name
   * @return string
   */
  public function getJavascriptFunction(GhGuiComponent $source, $event)
  {
    if($this->_form)
      $form = $this->_form;
    else
    {
      $form = $source;

      while($form && !($form instanceof GhGuiForm))
        $form = $form->getParentContainer();
    }

    if(!$form)
      throw new GhGuiException('GhGuiEventSetHiddenField event can only be registered on components that are contained within a GhGuiForm');

    if($this->_name instanceof GhGuiHiddenField)
      $hiddenField = $this->_name;
    else
    {
      $hiddenField = $form->getHiddenField($this->_name);

      if(!($hiddenField instanceof GhGuiHiddenField))
        throw new GhGuiException('GhGuiEventSetHiddenField could not find a hidden field "' . $this->_name . '" within the form');
    }

    if($this->_valueIsJavascript)
      $value = $this->_value;
    else
      $value = GhCoreStrings::attributeValue($this->_value);

    $javascript = 'function(event){var value=' . $hiddenField->getJavascriptWidgetVariable() . '.value;var newValue=' . $value . ';if(newValue!=undefined)' . $hiddenField->getJavascriptWidgetVariable() . '.value=newValue}';

    return $javascript;
  }
}