<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * GUI Hilfsklasse.
 *
 * Diese Klasse enthält einige Einstellungen und Methoden für die Ausgabe des
 * Webseiteninhalts.
 *
 * @package GhGui
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGui
{
  /**
   * DOCTYPE für XHTML 1.0 strict.
   */
  const DOCTYPE_XHTML_1_STRICT = 'xhtml1 strict';

  /**
   * DOCTYPE für XHTML 1.0 transitional.
   */
  const DOCTYPE_XHTML_1_TRANSITIONAL = 'xhtml1 transitional';

  /**
   * DOCTYPE für HTML 4.01 strict.
   */
  const DOCTYPE_HTML_4_STRICT = 'html4 strict';

  /**
   * DOCTYPE für HTML 4.01 transitional.
   */
  const DOCTYPE_HTML_4_TRANSITIONAL = 'html4 transitional';

  /**
   * Prefix für alle DOM IDs die über die createId() Methode erzeugt werden.
   * @var string
   */
  static protected $_componentIdPrefix = 'ghgui';

  /**
   * Ausgabeformat (DOCTYPE) für die Webseite (Vorgabewert: XHTML 1.0 strict).
   * @var string
   */
  static protected $_docType = self::DOCTYPE_XHTML_1_STRICT;

  /**
   * Zeichencodierung.
   * Die Voreinstellung ist 'UTF-8'.
   * @var string
   */
  static protected $_charset = 'UTF-8';

  /**
   * Basis-URL der Webseite.
   * Dies ist die URL, unter der der 'lib'-Ordner liegt (in dem sich die
   * GREYHOUND-Javascripte und -Stylesheets befinden).
   * @var string
   */
  static protected $_baseUrl = '/';

  /**
   * Autoload-Verzeichnisse.
   * Diese Liste wird beim Erzeugen der Klasse automatisch befüllt und kann
   * über die Methode addAutoLoadPath() ergänzt werden, um eigene Verzeichnisse
   * zum Autoloader hinzuzufügen.
   * @var array
   */
  static protected $_autoLoadPaths = array();

  /**
   * Debug-Modus.
   * @var boolean
   */
  static protected $_debugMode = false;

  /**
   * Zähler für die Erzeugung eindeutiger IDs.
   * @var int
   */
  static protected $_uniqueIdCount = 0;

  /**
   * Zeilenende.
   * Im Debug-Modus werden die Zeilen durch einen Zeilenumbruch getrennt,
   * andernfalls werden Zeilen nicht getrennt.
   * @var string
   */
  static public $eol = '';

  /**
   * Legt fest, ob der Debug-Modus aktiv ist.
   *
   * @param boolean $debug true: Debug-Modus aktivieren, false: deaktivieren
   */
  static public function setDebugMode($debug)
  {
    self::$_debugMode = $debug;
    self::$eol = $debug ? "\r\n" : '';
  }

  /**
   * Liefert den Status des Debug-Modus.
   *
   * @return boolean
   */
  static public function getDebugMode()
  {
    return self::$_debugMode;
  }

  /**
   * Setzt die Zeichencodierung der Webseite.
   *
   * @param string $charset Zeichensatz
   */
  static public function setCharset($charset)
  {
    self::$_charset = $charset;
  }

  /**
   * Liefert die Zeichencodierung der Webseite.
   *
   * @return string
   */
  static public function getCharset()
  {
    return self::$_charset;
  }

  /**
   * Setzt die Basis-URL für die Webseite.
   *
   * Dies ist die URL, unter der der 'lib'-Ordner liegt (in dem sich die
   * GREYHOUND-Javascripte und -Stylesheets befinden).
   *
   * @param string  $url    URL
   * @param boolean $fixUrl true: Slash ('/') an URL anhängen, falls diese
   *   nicht mit einem Slash endet, false: URL unverändert übernehmen
   */
  static public function setBaseUrl($url, $fixUrl = true)
  {
    self::$_baseUrl = $url;

    if($fixUrl)
      self::$_baseUrl = rtrim(self::$_baseUrl, '/') . '/';
  }

  /**
   * Liefert die Basis-URL für die Webseite.
   *
   * @return string
   */
  static public function getBaseUrl()
  {
    return self::$_baseUrl;
  }

  /**
   * Erzeugt eine für diese Webseite geeignete URL.
   *
   * Beginnt die URL mit 'ghgui://', so wird dies durch die Basis-URL der
   * Webseite ersetzt (siehe setBaseUrl()).
   *
   * Beispiel: die URL 'ghgui://lib/greyhound/greyhound.js' wird zu
   * '/Web/lib/greyhound/greyhound.js' wenn die Standardeinstellung für die
   * Basis-URL ('/Web/') nicht verändert wurde.
   *
   * @param string $url URL
   * @return string
   */
  static public function createUrl($url)
  {
    if(substr($url, 0, 8) == 'ghgui://')
      $url = self::$_baseUrl . substr($url, 8);
    return $url;
  }

  /**
   * Autoload-Funktion für Klassen.
   *
   * @param string $class Klassenname
   */
  static public function autoLoadClass($class)
  {
    $file = strtolower($class) . '.php';
    foreach(self::$_autoLoadPaths as $path)
    {
      $filePath = $path . '/' . $file;
      if(file_exists($filePath))
      {
        require_once $filePath;
        return;
      }
    }
  }

  /**
   * Fügt einen Pfad zum Autoloader hinzu.
   *
   * @throws GhGuiException falls der übergebene Pfad nicht existiert
   *
   * @param string $path Verzeichnispfad für den Autoloader
   */
  static public function addAutoLoadPath($path)
  {
    if(!file_exists($path))
      throw new GhGuiException('The autoload path "' . $path . '" does not exist');

    self::$_autoLoadPaths[] = rtrim($path, '/\\') . '/';
  }

  /**
   * Erzeugt eine eindeutige ID, die für DOM Knoten verwendet werden kann.
   *
   * Die ID wird aus einem Prefix und einer eindeutigen ID zusammengesetzt.
   *
   * @return string Eindeutige ID
   */
  static public function createId()
  {
    return uniqid(self::$_componentIdPrefix) . (int)rand(100000, 999999) . (++self::$_uniqueIdCount);
  }

  /**
   * Setzt den DOCTYPE der Webseite.
   *
   * @param string $docType DOCTYPE (z.B. eine der GhGui::DOCTYPE_* Konstanten)
   */
  static public function setDocType($docType)
  {
    self::$_docType = $docType;
  }

  /**
   * Liefert den DOCTYPE, der für die Ausgabe der Webseite verwendet wird.
   *
   * @return string DOCTYPE
   */
  static public function getDocType()
  {
    return self::$_docType;
  }

  /**
   * Liefert den Code für den Webseiten-Anfang.
   *
   * Die Methode liefert den DOCTYPE und das öffnende HTML-Tag.
   *
   * @param string $docType DOCTYPE (null: den DOCTYPE der GhGui verwenden)
   * @param string $charset Zeichencodierung (null: die Zeichencodierung der GhGui verwenden)
   * @return string Seitenanfangs-Code
   */
  static public function createHtmlStart($docType = null, $charset = null)
  {
    if($docType === null)
      $docType = GhGui::getDocType();

    if($charset === null)
      $charset = GhGui::getCharset();

    switch($docType)
    {
      case self::DOCTYPE_XHTML_1_STRICT:
        return '<?xml version="1.0" encoding="' . $charset . '" ?>' . "\n"
               . '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
               . "\n" . '<html xmlns="http://www.w3.org/1999/xhtml">' . "\n";

      case self::DOCTYPE_XHTML_1_TRANSITIONAL:
        return '<?xml version="1.0" encoding="' . $charset . '" ?>' . "\n"
               . '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' . "\n"
               . '<html xmlns="http://www.w3.org/1999/xhtml">' . "\n";

      case self::DOCTYPE_HTML_4_STRICT:
        return '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">' . "\n"
               . '<html>' . "\n";

      case self::DOCTYPE_HTML_4_TRANSITIONAL:
        return '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">' . "\n"
               . '<html>' . "\n";

      default:
        return '<!DOCTYPE html>' . "\n"
               . '<html>' . "\n";
    }
  }

  /**
   * Liefert den Code für das Webseiten-Ende.
   *
   * Die Methode liefert das schließende HTML-Tag.
   *
   * @return string Seitenende-Code
   */
  static public function createHtmlEnd()
  {
    return "\n" . '</html>' . "\n";
  }

  /**
   * Erzeugt HTML-Code für einen DOM Knoten.
   *
   * Der HTML-Code hängt vom verwendeten DOCTYPE ab.
   *
   * @param string  $tag        HTML Knotentyp (Tag Name)
   * @param array   $attributes Attribute für den HTML Knoten
   * @param string  $content    Inhalt für den HTML Knoten
   * @param boolean $xhtml      true: XHTML generieren, false: HTML generieren, null: DOCTYPE der GhGui verwenden um festzulegen ob XHTML generiert wird
   * @return string
   */
  static public function createHtml($tag, $attributes = null, $content = null, $xhtml = null)
  {
    if($xhtml === null)
      $xhtml = GhGui::isXml();

    $tag = strtolower($tag);

    $html = '<' . $tag;

    if(is_array($attributes))
    {
      foreach($attributes as $attribute => $value)
      {
        if(strlen($attribute) > 0 && $value !== null)
          $html .= ' ' . $attribute . '="' . GhCoreStrings::attributeValue($value) . '"';
      }
    }

    // XHTML verwendet Short-Tags, HTML verwendet öffnende und schließende Tags:
    if(!empty($content) || $content === '0')
    {
      // Javascript in XHTML in CDATA einschließen:
      if(($tag == 'script' || $tag == 'style') && $xhtml)
        $html .= '>/* <![CDATA[ */' . GhGui::$eol . $content . GhGui::$eol . '/* ]]> */</' . $tag . '>' . GhGui::$eol;
      else
        $html .= '>' . $content . '</' . $tag . '>' . GhGui::$eol;
    }
    else
    {
      // Korrekte Behandlung von Short-Tags:
      switch($tag)
      {
        case 'area':
        case 'base':
        case 'br':
        case 'col':
        case 'hr':
        case 'img':
        case 'input':
        case 'link':
        case 'meta':
        case 'param':
          if($xhtml)
            $html .= ' />' . GhGui::$eol;
          else
            $html .= '>' . GhGui::$eol;
          break;
        default:
          $html .= '></' . $tag . '>' . GhGui::$eol;
          break;
      }
    }

    return $html;
  }

  /**
   * Prüft, ob die Webseitenausgabe XML-konform sein soll oder nicht.
   *
   * XML-konforme Webseiten (XHTML) schließen Short-Tags mit einem '/' am Ende des Tags,
   * bei HTML wird der '/' weggelassen.
   *
   * Liefert true falls die Webseitenausgabe XML-konform sein soll und
   * false wenn sie HTML- (und nicht XML-) konform sein soll.
   *
   * @param string $docType DOCTYPE (null: der DOCTYPE der GhGui wird verwendet)
   * @return boolean
   */
  static public function isXml($docType = null)
  {
    if($docType === null)
      $docType = GhGui::getDocType();

    return $docType == self::DOCTYPE_XHTML_1_STRICT || $docType == self::DOCTYPE_XHTML_1_TRANSITIONAL;
  }
}

// Initialisierung:
$ghGuiBaseDir = realpath(dirname(__FILE__)) . '/';
GhGui::addAutoLoadPath($ghGuiBaseDir . 'core');
GhGui::addAutoLoadPath($ghGuiBaseDir . 'component');
GhGui::addAutoLoadPath($ghGuiBaseDir . 'container');
GhGui::addAutoLoadPath($ghGuiBaseDir . 'event');
GhGui::addAutoLoadPath($ghGuiBaseDir . 'exception');
GhGui::addAutoLoadPath($ghGuiBaseDir . 'formatter');
GhGui::addAutoLoadPath($ghGuiBaseDir . 'page');
GhGui::addAutoLoadPath(dirname($ghGuiBaseDir) . '/ghcore');

// Registriert die Autoload-Funktion des GhGui-Singletons:
spl_autoload_register(array('GhGui', 'autoLoadClass'));

// Übernimmt die Base-URL aus einer Konstante (sofern diese gesetzt wurde):
if(defined('GREYHOUND_PHP_GUI_BASE_URL'))
  GhGui::setBaseUrl(GREYHOUND_PHP_GUI_BASE_URL);
