/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2020 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

// Namespace:
var greyhound = new function(){
  this.baseUrl = '';
}();

/**
 * Utility functions for the GREYHOUND PHP API.
 */
greyhound.dojo = new function()
{
  /**
   * Finds and returns an item in the tree by its property value.
   *
   * @param dijit.Tree tree     tree object
   * @param string     property item property name
   * @param string     value    item property value
   * @return object tree item with the given value in the given property,
   *   or undefined if the item could not be found
   */
  this.getTreeItem = function(tree, property, value)
  {
    return _getTreeItemRecursive(tree, tree.model.root, property, value);
  };

  var _getTreeItemRecursive = function(tree, item, property, value)
  {
    if(item && item[property] == value)
      return item;

    for(var childIndex in item.children)
    {
      var result = _getTreeItemRecursive(tree, item.children[childIndex], property, value);
      if(result)
        return result;
    }
    return undefined;
  };

  /**
   * Finds an item in the tree by its property value and returns the item's
   * path as an array of ids (e.g. for setting the tree path attribute to
   * select the item).
   *
   * @param dijit.Tree tree     tree object
   * @param string     property item property name
   * @param string     value    item property value
   * @return Array path of item ids, or undefined if the item could not be found
   */
  this.getTreePath = function(tree, property, value)
  {
    return _getTreePathRecursive(tree, tree.model.root, property, value);
  };

  var _getTreePathRecursive = function(tree, item, property, value)
  {
    if(item && item[property] == value)
      return new Array(tree.model.getIdentity(item));

    for(var childIndex in item.children)
    {
      var result = _getTreePathRecursive(tree, item.children[childIndex], property, value);
      if(result)
      {
        result.unshift(tree.model.getIdentity(item));
        return result;
      }
    }
    return undefined;
  };

  /**
   * Sets the selection in a dojox.grid.DataGrid widget according to the
   * selected options in a hidden select node.
   *
   * @param dojox.grid.DataGrid grid        data grid widget
   * @param object              hiddenField hidden DOM select node
   */
  this.setDataGridSelection = function(grid, hiddenField)
  {
    grid.selection.clear();
    dojo.query("> option[selected=\"selected\"]", dojo.byId(hiddenField)).forEach(function(node, index, nodeList)
    {
      grid.store.fetchItemByIdentity({
        identity: node.innerHTML,
        onItem: function(item, request)
        {
          grid.selection.addToSelection(grid.getItemIndex(item));
        }
      });
    });
  };

  /**
   * Selects and deselects the options in a hidden select node according to the
   * selection in a dojox.grid.DataGrid.
   *
   * @param dojox.grid.DataGrid grid        data grid widget
   * @param object              hiddenField hidden DOM select node
   */
  this.updateDataGridSelectionHiddenField = function(grid, hiddenField)
  {
    dojo.query("> option", dojo.byId(hiddenField)).forEach(function(node, index, nodeList)
    {
      node.selected = false;
    });

    var selection = grid.selection.getSelected();
    if(selection)
    {
      for(i=0; i<selection.length; i++)
      {
        var option = dojo.byId("option_" + selection[i]._id);
        if(option)
          option.selected = true;
      }
    }
  };

  /**
   * Increases the size of a node to fill up the remaining space of its parent.
   *
   * @param object  node       DOM node
   * @param boolean fillWidth  true: fill remaining width
   * @param boolean fillHeight true: fill remaining height
   */
  this.fillSize = function(node, fillWidth, fillHeight)
  {
    if(node == undefined || node.parentNode == undefined)
      return;

    var parentNode = node.parentNode;
    var parentSize = dojo.contentBox(parentNode);

    // Only resize if the parent node is visible:
    if(!parentSize || parentSize.l < 0 || parentSize.t < 0 || parentSize.w <= 0 || parentSize.h <= 0)
      return;

    // Minimize node to prevent unnecessary wrapping if the node would be too large:
    if(fillWidth)
      node.style.width = "8px";

    if(fillHeight)
      node.style.height = "8px";

    var nodeSize = dojo.contentBox(node);
    var nodePos = dojo.marginBox(node);

    var right = null;
    var bottom = null;
    var display = dojo.style(node,"display");

    // Find the rightmost and bottommost nodes:
    for(var i=0; i<parentNode.childNodes.length; i++)
    {
      var childSize = dojo.marginBox(parentNode.childNodes[i]);

      if(!childSize)
        continue;

      if(childSize.w > 0 && (!right || right.l + right.w < childSize.l + childSize.w))
        right = childSize;

      if(childSize.h > 0 && (!bottom || bottom.t + bottom.h < childSize.t + childSize.h))
        bottom = childSize;
    }

    // Change node width:
    if(fillWidth && right)
    {
      var w = nodeSize.w + parentSize.l + parentSize.w - right.l - right.w;

      // Since inline tables don't seem to add their border widths, just add them to the node width:
      if(display == "inline-table")
        w += dojo.style(node,"border-left-width") + dojo.style(node,"border-right-width");

      if(w > 0 && w > nodeSize.w)
        node.style.width = w + "px";
    }

    // Change node height:
    if(fillHeight && bottom)
    {
      var h = nodeSize.h + parentSize.t + parentSize.h - right.t - right.h;

      // Since inline tables don't seem to add their border widths, just add them to the node height:
      if(display == "inline-table")
        h += dojo.style(node,"border-top-width") + dojo.style(node,"border-bottom-width");

      if(h > 0 && h > nodeSize.h)
        node.style.height = h + "px";
    }
  };
}();
